# copyright 2000, greg landrum
"""

Here we're implementing the Decision Tree stuff found in Chapter 3 of
Tom Mitchell's Machine Learning Book.

"""
from warnings import warn

warn('This module is deprecated and will be removed in the 2024.03 release', DeprecationWarning,
     stacklevel=2)



import unittest

from rdkit import Chem
from rdkit.Chem.MolStandardize.standardize import Standardizer


class FakeStandardizer(Standardizer):

  def normalize(self):

    def fake_normalize(y):
      props = y.GetPropsAsDict()
      for k, v in props:
        y.ClearProp(k)
      return y

    return fake_normalize


class TestCase(unittest.TestCase):

  def testPreserveProps(self):
    PROP_NAME = "MyProp"
    PROP_VALUE = "foo"
    standardizer = FakeStandardizer()
    m = Chem.MolFromSmiles("C")
    m.SetProp(PROP_NAME, PROP_VALUE)

    standardized_mol = standardizer.standardize(m)
    self.assertTrue(standardized_mol.HasProp(PROP_NAME))
    self.assertEqual(PROP_VALUE, standardized_mol.GetProp(PROP_NAME))



# Run this with:
#  python setup.py install --install-lib=.
from distutils.core import Extension, setup

from RDBuild import *

setup(
  name="SPtrTestModule", version="1.0", ext_modules=[
    Extension("SPtrTestModule", ["module.cpp"], library_dirs=libDirs, libraries=libraries,
              extra_link_args=linkArgs, extra_compile_args=compileArgs)
  ])



#
#  Copyright (C) 2001  greg Landrum
#

# unit testing code for the composite model COM server

import unittest

from rdkit import RDConfig

try:
  from win32com.client import Dispatch
except ImportError:
  Dispatch = None


@unittest.skipIf(Dispatch is None, 'Test for Windows only')
class TestCase(unittest.TestCase):

  def testConnect(self):
    # connecting to COM server
    ok = 1
    try:
      c = Dispatch('RD.Composite')
    except Exception:
      ok = 0
    assert ok and c is not None, 'connection to COM server failed'

  def testLoad(self):
    # loading a composite
    c = Dispatch('RD.Composite')
    ok = 1
    try:
      c.LoadComposite(RDConfig.RDCodeDir + '/ml/composite/test_data/composite_base.pkl')
    except Exception:
      ok = 0
    assert ok, 'LoadComposite failed'

  def testNames(self):
    # testing descriptor names
    c = Dispatch('RD.Composite')
    c.LoadComposite(RDConfig.RDCodeDir + '/ml/composite/test_data/composite_base.pkl')
    names = c.GetDescriptorNames()
    expectedNames = ('composition', 'max_atomic', 'has3d', 'has4d', 'has5d', 'elconc', 'atvol',
                     'isferro')
    assert names == expectedNames, 'GetDescriptorNames failed'

  def testInputOrder(self):
    # testing input order
    c = Dispatch('RD.Composite')
    c.LoadComposite(RDConfig.RDCodeDir + '/ml/composite/test_data/composite_base.pkl')
    names = c.GetDescriptorNames()
    ok = 1
    try:
      c.SetInputOrder(names)
    except Exception:
      ok = 0
    assert ok, 'SetInputOrder failed'

  def testClassify(self):
    # testing classification
    argV = [
      'CrPt3', 'fcc', 'AuCu3', 58.09549962, 36, 4, 0.228898, 2.219, 1, 3.67481803894, 1, 0, 1,
      0.619669341609, 14.523874905
    ]
    nameV = [
      'composition', 'Structure', 'Structure_Type', 'Volume', 'Electrons_Per_Unit',
      'Atoms_Per_Unit', 'Hardness', 'DOS_Ef', 'isferro', 'max_atomic', 'has3d', 'has4d', 'has5d',
      'elconc', 'atvol'
    ]
    c = Dispatch('RD.Composite')
    c.LoadComposite(RDConfig.RDCodeDir + '/ml/composite/test_data/composite_base.pkl')
    c.SetInputOrder(nameV)
    res = c.ClassifyExample(argV)
    expected = [1, 1.0]
    assert res[0] == expected[0], 'bad prediction'
    assert res[1] == expected[1], 'bad confidence'


def TestSuite():  # pragma: nocover
  suite = unittest.TestSuite()
  suite.addTest(TestCase('testConnect'))
  suite.addTest(TestCase('testLoad'))
  suite.addTest(TestCase('testNames'))
  suite.addTest(TestCase('testInputOrder'))
  suite.addTest(TestCase('testClassify'))
  return suite


if __name__ == '__main__':  # pragma: nocover
  suite = TestSuite()
  unittest.TextTestRunner().run(suite)



import doctest
import gzip
import os
import shutil
import tempfile
import unittest
from io import BytesIO, StringIO

import numpy

from rdkit import Chem, RDConfig, rdBase
from rdkit.Chem import PandasTools

if (getattr(doctest, 'ELLIPSIS_MARKER')):
  doctest.ELLIPSIS_MARKER = '*...*'

try:
  import IPython
except ImportError:
  IPython = None

# We make sure that we don't mess up the Mol methods for the rest of the tests
PandasTools.UninstallPandasTools()


@unittest.skipIf((not hasattr(PandasTools, 'pd')) or PandasTools.pd is None,
                 'Pandas not installed, skipping')
class TestPandasTools(unittest.TestCase):

  def __init__(self, methodName='runTest'):
    self.df = None
    super(TestPandasTools, self).__init__(methodName=methodName)

  def initialize_dataframe(self):
    # We only need to initialize the dataframe once, but we defer to actual running of the tests,
    # as getTestFrame() needs pandas installed, and __init__() is run even in the absence of pandas.
    if self.df is None:
      self.df = getTestFrame()
      self.df.index.name = 'IndexName'

  def setUp(self):
    self.initialize_dataframe()
    PandasTools.InstallPandasTools()
    PandasTools.ChangeMoleculeRendering(renderer='PNG')
    PandasTools.pd.set_option('display.max_columns', None)
    self._molRepresentation = PandasTools.molRepresentation
    self._highlightSubstructures = PandasTools.highlightSubstructures

  def tearDown(self):
    PandasTools.molRepresentation = self._molRepresentation
    PandasTools.highlightSubstructures = self._highlightSubstructures
    PandasTools.UninstallPandasTools()

  def testDoctest(self):
    # We need to do it like this to ensure that default RDkit functionality is restored
    failed, _ = doctest.testmod(PandasTools,
                                optionflags=doctest.ELLIPSIS + doctest.NORMALIZE_WHITESPACE)
    self.assertFalse(failed)

  def test_RestoreMonkeyPatch(self):
    sio = getStreamIO(methane + peroxide)
    df = PandasTools.LoadSDF(sio)
    html = df.to_html()
    self.assertIn('data:image/png;base64', html)
    self.assertIn('table', html)

    PandasTools.UninstallPandasTools()
    html = df.to_html()
    self.assertNotIn('data:image/png;base64', html)
    self.assertIn('rdkit.Chem.rdchem.Mol', html)
    self.assertIn('table', html)

    PandasTools.InstallPandasTools()
    html = df.to_html()
    self.assertIn('data:image/png;base64', html)
    self.assertIn('table', html)

    PandasTools.UninstallPandasTools()
    html = df.to_html()
    self.assertNotIn('data:image/png;base64', html)
    self.assertIn('rdkit.Chem.rdchem.Mol', html)
    self.assertIn('table', html)

  def test_FrameToGridImage(self):
    # This test only makes sure that we get no exception. To see the created images, set
    # interactive to True
    interactive = False
    self.assertTrue(True)
    df = self.df

    result = PandasTools.FrameToGridImage(df)
    if interactive:
      result.show()

    result = PandasTools.FrameToGridImage(df, legendsCol='PUBCHEM_IUPAC_INCHIKEY')
    if interactive:
      result.show()

    result = PandasTools.FrameToGridImage(df, legendsCol=df.index.name)
    if interactive:
      result.show()

  def test_AddMurckoToFrame(self):
    df = self.df.copy()
    self.assertIn('ROMol', df.columns)
    self.assertNotIn('Murcko_SMILES', df.columns)
    PandasTools.AddMurckoToFrame(df)
    self.assertIn('ROMol', df.columns)
    self.assertIn('Murcko_SMILES', df.columns)
    self.assertEqual(df['Murcko_SMILES'][10], 'O=C(CCn1c(-c2ccccc2)n[nH]c1=S)Nc1ccccn1')

    PandasTools.AddMurckoToFrame(df, Generic=True)
    self.assertIn('ROMol', df.columns)
    self.assertIn('Murcko_SMILES', df.columns)
    self.assertEqual(df['Murcko_SMILES'][10], 'CC(CCC1C(C)CCC1C1CCCCC1)CC1CCCCC1')

  def test_SaveSMILESFromFrame(self):
    sio = StringIO()
    PandasTools.SaveSMILESFromFrame(self.df, sio)
    result = sio.getvalue()
    self.assertIn(self.df['SMILES'][10], result)
    self.assertIn(self.df['ID'][10], result)

    sio = StringIO()
    PandasTools.SaveSMILESFromFrame(self.df, sio, NamesCol='PUBCHEM_IUPAC_INCHIKEY')
    result = sio.getvalue()
    self.assertIn(self.df['SMILES'][10], result)
    self.assertIn(self.df['PUBCHEM_IUPAC_INCHIKEY'][10], result)

  @unittest.skipIf(IPython is None, 'Package IPython required for testing')
  def test_svgRendering(self):
    df = PandasTools.LoadSDF(getStreamIO(methane + peroxide))
    self.assertIn('image/png', df.to_html())
    self.assertNotIn('svg', df.to_html())

    PandasTools.molRepresentation = 'svg'
    self.assertIn('svg', df.to_html())
    self.assertNotIn('image/png', df.to_html())

    # we can use upper case for the molRepresentation
    PandasTools.molRepresentation = 'PNG'
    self.assertNotIn('svg', df.to_html())
    self.assertIn('image/png', df.to_html())

  def test_patchHeadFrame(self):
    df = self.df.copy()
    result = str(df.head())
    self.assertIn('35024984', result)
    self.assertNotIn('35024985', result)

  def test_AddMoleculeColumnToFrame(self):
    df = PandasTools.LoadSDF(getStreamIO(methane + peroxide), isomericSmiles=True,
                             smilesName='Smiles')
    PandasTools.ChangeMoleculeRendering(frame=df, renderer='String')
    del df['ROMol']
    self.assertNotIn('ROMol', str(df))
    PandasTools.AddMoleculeColumnToFrame(df, includeFingerprints=False)
    self.assertIn('ROMol', str(df))

  def test_molge(self):
    # We want to have the default RDkit functionality for testing
    PandasTools.UninstallPandasTools()
    molge = PandasTools._molge
    mol1 = Chem.MolFromSmiles('CCC')
    mol2 = Chem.MolFromSmiles('CC')
    mol3 = Chem.MolFromSmiles('CN')

    self.assertFalse(molge(mol1, None))
    self.assertFalse(molge(None, mol1))

    self.assertFalse(hasattr(mol1, '_substructfp'))
    self.assertFalse(hasattr(mol2, '_substructfp'))
    self.assertFalse(hasattr(mol1, '__sssAtoms'))
    self.assertFalse(hasattr(mol2, '__sssAtoms'))

    self.assertTrue(molge(mol1, mol2))
    self.assertEqual(mol1.__dict__['__sssAtoms'], [0, 1])
    PandasTools.highlightSubstructures = False
    self.assertTrue(molge(mol1, mol2))
    self.assertEqual(mol1.__dict__['__sssAtoms'], [])

    PandasTools.highlightSubstructures = True
    self.assertFalse(molge(mol2, mol1))
    self.assertEqual(mol2.__dict__['__sssAtoms'], [])

    self.assertFalse(molge(mol1, mol3))
    self.assertEqual(mol1.__dict__['__sssAtoms'], [])

  @unittest.skipIf(IPython is None, 'Package IPython required for testing')
  def test_github2380(self):
    from rdkit.Chem.Draw import IPythonConsole
    IPythonConsole.ipython_useSVG = True
    df = PandasTools.LoadSDF(getStreamIO(methane + peroxide))
    _ = PandasTools.FrameToGridImage(df)

  def test_RGD(self):
    from rdkit.Chem import rdRGroupDecomposition
    scaffold = Chem.MolFromSmiles('c1ccccn1')
    mols = [
      Chem.MolFromSmiles(smi)
      for smi in 'c1c(F)cccn1 c1c(Cl)c(C)ccn1 c1c(O)cccn1 c1c(F)c(C)ccn1 c1cc(Cl)c(F)cn1'.split()
    ]
    groups, _ = rdRGroupDecomposition.RGroupDecompose([scaffold], mols, asSmiles=True, asRows=False)
    df = PandasTools.RGroupDecompositionToFrame(groups, mols, include_core=True)
    self.assertEqual(len(df), len(mols))
    self.assertEqual(list(df.columns), ['Mol', 'Core', 'R1', 'R2'])
    self.assertEqual(list(df.R2), ['F[*:2]', 'Cl[*:2]', 'O[*:2]', 'F[*:2]', 'F[*:2]'])

    groups, _ = rdRGroupDecomposition.RGroupDecompose([scaffold], mols, asSmiles=False,
                                                      asRows=False)
    df = PandasTools.RGroupDecompositionToFrame(groups, mols, include_core=True)
    self.assertEqual(len(df), len(mols))
    self.assertEqual(list(df.columns), ['Mol', 'Core', 'R1', 'R2'])
    self.assertEqual([Chem.MolToSmiles(x) for x in df.R2],
                     ['F[*:2]', 'Cl[*:2]', 'O[*:2]', 'F[*:2]', 'F[*:2]'])

  def testPandasShouldShowMoleculesWhenTruncating(self):
    csv_data = '''"Molecule ChEMBL ID";"Molecule Name";"Molecule Max Phase";"Molecular Weight";"#RO5 Violations";"AlogP";"Compound Key";"Smiles";"Standard Type";"Standard Relation";"Standard Value";"Standard Units";"pChEMBL Value";"Data Validity Comment";"Comment";"Uo Units";"Ligand Efficiency BEI";"Ligand Efficiency LE";"Ligand Efficiency LLE";"Ligand Efficiency SEI";"Potential Duplicate";"Assay ChEMBL ID";"Assay Description";"Assay Type";"BAO Format ID";"BAO Label";"Assay Organism";"Assay Tissue ChEMBL ID";"Assay Tissue Name";"Assay Cell Type";"Assay Subcellular Fraction";"Target ChEMBL ID";"Target Name";"Target Organism";"Target Type";"Document ChEMBL ID";"Source ID";"Source Description";"Document Journal";"Document Year";"Cell ChEMBL ID"
  "CHEMBL543779";"";"0";"341.86";"0";"2.60";"1w";"CCN(CC)CCS/C(=N\O)C(=O)c1ccc(C#N)cc1.Cl";"IC50";"'='";"180000.0";"nM";"";"Outside typical range";"";"UO_0000065";"";"";"";"";"False";"CHEMBL644102";"Reversible inhibition of Human AchE";"B";"BAO_0000357";"single protein format";"None";"None";"None";"None";"None";"CHEMBL220";"Acetylcholinesterase";"Homo sapiens";"SINGLE PROTEIN";"CHEMBL1123431";"1";"Scientific Literature";"J. Med. Chem.";"1986";"None"
  '''
    try:
      with StringIO(csv_data) as hnd:
        df = PandasTools.pd.read_csv(hnd, sep=";")
        PandasTools.InstallPandasTools()
        PandasTools.RenderImagesInAllDataFrames()
        PandasTools.AddMoleculeColumnToFrame(df, 'Smiles')
        html_output = df.to_html(notebook=True, max_cols=10)
        self.assertIn('...', html_output)
        self.assertIn('data-content="rdkit/molecule"', html_output)
        self.assertIn('data:image/png;base64', html_output)
    finally:
      PandasTools.UninstallPandasTools()


@unittest.skipIf((not hasattr(PandasTools, 'pd')) or PandasTools.pd is None,
                 'Pandas not installed, skipping')
class TestLoadSDF(unittest.TestCase):
  gz_filename = os.path.join(RDConfig.RDCodeDir, 'Chem', 'test_data', 'pandas_load.sdf.gz')

  # the doctest tests loading from a ".sdf" file so there's no need for that test here

  def test_load_gzip_file(self):
    rdBase.DisableLog('rdApp.error')
    df = PandasTools.LoadSDF(self.gz_filename)
    rdBase.EnableLog('rdApp.error')
    self.assertEqual(len(df), 13)
    # The molecule with index 1 is invalid, so it should be missing form the index
    self.assertEqual(list(df.index), [0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13])

  def test_load_from_sio(self):
    sio = getStreamIO(methane + peroxide)
    df = PandasTools.LoadSDF(sio)
    self.assertEqual(len(df), 2)
    self.assertEqual(list(df["ID"]), ["Methane", "Peroxide"])
    atom_counts = [mol.GetNumAtoms() for mol in df["ROMol"]]
    self.assertEqual(atom_counts, [1, 2])

  def test_load_specify_column_names(self):
    sio = getStreamIO(methane + peroxide)
    df = PandasTools.LoadSDF(sio, idName="CorpID", molColName="_rdmol")
    self.assertEqual(len(df), 2)
    self.assertEqual(list(df["CorpID"]), ["Methane", "Peroxide"])
    atom_counts = [mol.GetNumAtoms() for mol in df["_rdmol"]]
    self.assertEqual(atom_counts, [1, 2])

  def test_empty_file(self):
    # Should return an empty data frame with no rows or columns
    sio = getStreamIO(None)
    df = PandasTools.LoadSDF(sio)
    self.assertEqual(len(df), 0)
    self.assertEqual(len(df.index), 0)

  def test_passed_in_file_is_not_closed(self):
    sio = getStreamIO(methane)
    df = PandasTools.LoadSDF(sio)
    self.assertEqual(len(df), 1)
    self.assertFalse(sio.closed)

  def test_properties(self):
    sio = getStreamIO(peroxide + methane)
    df = PandasTools.LoadSDF(sio)
    self.assertEqual(set(df.columns), set("ROMol ID prop1 prop2 prop3".split()))
    prop1 = list(df["prop1"])
    self.assertTrue(numpy.isnan(prop1[0]), prop1[0])
    self.assertEqual(prop1[1], "12.34")

    self.assertEqual(list(df["prop2"]), ["rtz", "qwe"])

    prop3 = list(df["prop3"])
    self.assertEqual(prop3[0], "yxcv")
    self.assertTrue(numpy.isnan(prop3[1]), prop3[1])

  def test_ignore_mol_column(self):
    sio = getStreamIO(peroxide + methane)
    df = PandasTools.LoadSDF(sio, molColName=None)
    self.assertEqual(set(df.columns), set("ID prop1 prop2 prop3".split()))


@unittest.skipIf((not hasattr(PandasTools, 'pd')) or PandasTools.pd is None,
                 'Pandas not installed, skipping')
class TestWriteSDF(unittest.TestCase):

  def setUp(self):
    self.df = PandasTools.LoadSDF(getStreamIO(methane + peroxide))

  def test_default_write_does_not_include_tags(self):
    sio = StringIO()
    PandasTools.WriteSDF(self.df, sio)
    s = sio.getvalue()
    self.assertNotIn(s, "prop2")

  def test_identifier_from_a_column(self):
    sio = StringIO()
    PandasTools.WriteSDF(self.df, sio, idName="prop2")
    s = sio.getvalue()
    first_line = s.split("\n", 1)[0]
    self.assertEqual(first_line, "qwe")

  def test_all_numeric_with_no_numeric_columns(self):
    sio = StringIO()
    PandasTools.WriteSDF(self.df, sio, allNumeric=True)
    s = sio.getvalue()
    self.assertFalse(">" in s, s)
    self.assertNotIn("7\n\n", s)  # double-check that the numeric tests don't pass by accident
    self.assertNotIn("8\n\n", s)

  def test_all_numeric_with_numeric_columns(self):
    sio = StringIO()
    df = self.df
    df["len"] = df["ID"].map(len)
    PandasTools.WriteSDF(df, sio, allNumeric=True)
    s = sio.getvalue()
    self.assertEqual(s.count("<len>"), 2)
    self.assertIn("7\n\n", s)
    self.assertIn("8\n\n", s)

  def test_specify_numeric_column(self):
    sio = StringIO()
    df = self.df
    df["len2"] = df["ID"].map(len)
    PandasTools.WriteSDF(df, sio, properties=["len2"])
    s = sio.getvalue()
    self.assertEqual(s.count("<len2>"), 2)
    self.assertIn("7\n\n", s)
    self.assertIn("8\n\n", s)

  def test_specify_numeric_column_2(self):
    sio = StringIO()
    df = self.df
    df["len2"] = df["ID"].map(len)
    df["len3"] = df["len2"].map(float)
    PandasTools.WriteSDF(df, sio, properties=["len2", "len3"])
    s = sio.getvalue()
    self.assertEqual(s.count("<len2>"), 2)
    self.assertEqual(s.count("<len3>"), 2)
    self.assertIn("7\n\n", s)
    self.assertIn("7.0\n\n", s)
    self.assertIn("8\n\n", s)
    self.assertIn("8.0\n\n", s)

  def test_write_to_sdf(self):
    dirname = tempfile.mkdtemp()
    try:
      filename = os.path.join(dirname, "test.sdf")
      PandasTools.WriteSDF(self.df, filename)
      with open(filename) as f:
        s = f.read()
      self.assertEqual(s.count("\n$$$$\n"), 2)
      self.assertEqual(s.split("\n", 1)[0], "Methane")

      # check file is V2000
      self.assertGreaterEqual(s.count("V2000"), 1)
      self.assertEqual(s.count("V3000"), 0)
    finally:
      shutil.rmtree(dirname)

  def test_write_V3000_to_sdf(self):
    dirname = tempfile.mkdtemp()
    try:
      filename = os.path.join(dirname, "test.sdf")
      PandasTools.WriteSDF(self.df, filename, forceV3000=True)
      with open(filename) as f:
        s = f.read()
      self.assertEqual(s.count("\n$$$$\n"), 2)
      self.assertEqual(s.split("\n", 1)[0], "Methane")

      # check file is V3000
      self.assertEqual(s.count("V2000"), 0)
      self.assertGreaterEqual(s.count("V3000"), 1)
    finally:
      shutil.rmtree(dirname)

  def test_write_to_sdf_gz(self):
    dirname = tempfile.mkdtemp()
    try:
      filename = os.path.join(dirname, "test.sdf.gz")
      PandasTools.WriteSDF(self.df, filename)
      with gzip.open(filename) as f:
        s = f.read()
      s = s.decode('utf-8')
      s = s.replace(os.linesep, '\n')
      self.assertEqual(s.count("\n$$$$\n"), 2)
      self.assertEqual(s.split("\n", 1)[0], "Methane")
    finally:
      shutil.rmtree(dirname)


def getStreamIO(sdfString):
  """ Return a StringIO/BytesIO for the string """
  sio = BytesIO() if sdfString is None else BytesIO(sdfString.encode('utf-8'))
  return sio


def getTestFrame():
  rdBase.DisableLog('rdApp.error')
  sdfFile = os.path.join(RDConfig.RDCodeDir, 'Chem', 'test_data', 'pandas_load.sdf.gz')
  df = PandasTools.LoadSDF(sdfFile, smilesName='SMILES')
  rdBase.EnableLog('rdApp.error')
  return df


methane = """\
Methane
     RDKit

  1  0  0  0  0  0  0  0  0  0999 V2000
    0.0000    0.0000    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
M  END
> <prop1>
12.34

> <prop2>
qwe

$$$$
"""

peroxide = """\
Peroxide
     RDKit

  2  1  0  0  0  0  0  0  0  0999 V2000
    0.0000    0.0000    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
    0.0000    0.0000    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
  1  2  1  0
M  END
> <prop2>
rtz

> <prop3>
yxcv

$$$$
"""

if __name__ == '__main__':  # pragma: nocover
  unittest.main()



import os
import sys
import unittest
from contextlib import contextmanager
from io import StringIO

from rdkit import RDConfig
from rdkit.Chem import FeatFinderCLI


class TestCase(unittest.TestCase):

  def test_FeatFinderCLI(self):
    smilesFile = os.path.join(RDConfig.RDDataDir, 'NCI', 'first_5K.smi')
    featureFile = os.path.join(RDConfig.RDCodeDir, 'Chem', 'Pharm2D', 'test_data',
                               'BaseFeatures.fdef')
    parser = FeatFinderCLI.initParser()
    cmd = '-n 10 {0} {1}'.format(featureFile, smilesFile)
    with outputRedirect() as (out, err):
      args = parser.parse_args(cmd.split())
      FeatFinderCLI.processArgs(args, parser)
    out = out.getvalue()
    err = err.getvalue()
    self.assertIn('Mol-1', out)
    self.assertIn('Acceptor-SingleAtomAcceptor', out)
    self.assertIn('C(1)', out)
    self.assertNotIn('Mol-11', out)
    self.assertEqual(err, '')

    cmd = '-n 2 -r {0} {1}'.format(featureFile, smilesFile)
    with outputRedirect() as (out, err):
      args = parser.parse_args(cmd.split())
      FeatFinderCLI.processArgs(args, parser)
    out = out.getvalue()
    err = err.getvalue()
    self.assertIn('Mol-1', out)
    self.assertIn('Acceptor-SingleAtomAcceptor:', out)
    self.assertIn('2, 3, 4', out)
    self.assertNotIn('Mol-3', out)
    self.assertEqual(err, '')

  def test_FeatFinderCLIexceptions(self):
    smilesFile = os.path.join(RDConfig.RDDataDir, 'NCI', 'first_5K.smi')
    featureFile = os.path.join(RDConfig.RDCodeDir, 'Chem', 'Pharm2D', 'test_data',
                               'BaseFeatures.fdef')
    parser = FeatFinderCLI.initParser()
    cmd = '-n 10 {0} {1}'.format(smilesFile, smilesFile)
    with self.assertRaises(SystemExit), outputRedirect() as (_, err):
      args = parser.parse_args(cmd.split())
      FeatFinderCLI.processArgs(args, parser)
    self.assertIn('error', err.getvalue())

    cmd = '-n 10 {0} {1}'.format(featureFile, 'incorrectFilename')
    with self.assertRaises(SystemExit), outputRedirect() as (_, err):
      args = parser.parse_args(cmd.split())
      FeatFinderCLI.processArgs(args, parser)
    self.assertIn('error', err.getvalue())


@contextmanager
def outputRedirect():
  """ Redirect standard output and error to String IO and return """
  try:
    _stdout, _stderr = sys.stdout, sys.stderr
    sys.stdout = sStdout = StringIO()
    sys.stderr = sStderr = StringIO()
    yield (sStdout, sStderr)
  finally:
    sys.stdout, sys.stderr = _stdout, _stderr


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



import copy
import math
import os
import pickle
import unittest

import numpy

import rdkit.DistanceGeometry as DG
from rdkit import Chem, RDConfig, rdBase
from rdkit.Chem import AllChem, ChemicalForceFields, rdDistGeom, rdMolAlign
from rdkit.Geometry import ComputeSignedDihedralAngle
from rdkit.Geometry import rdGeometry as geom
from rdkit.RDLogger import logger

logger = logger()


def feq(v1, v2, tol=1.e-4):
  return abs(v1 - v2) < tol


def lstEq(l1, l2, tol=1.0e-4):
  ln = len(l1)
  if (ln != len(l2)):
    return 0

  for i in range(ln):
    if abs(l1[i] - l2[i]) > tol:
      return 0
  return 1


def compareWithOld(smilesFile, sdFile):
  smiSup = Chem.SmilesMolSupplier(smilesFile, ",", 0, -1)
  sdsup = Chem.SDMolSupplier(sdFile)
  im = 0
  for mol in smiSup:
    cid = rdDistGeom.EmbedMolecule(mol, 10, 1)
    omol = sdsup[im]
    assert cid == 0
    conf = mol.GetConformer(0)
    oconf = omol.GetConformer()
    nat = mol.GetNumAtoms()
    for i in range(nat):
      #atm = mol.GetAtomWithIdx(i)
      #oatm = omol.GetAtomWithIdx(i)
      pos = conf.GetAtomPosition(i)
      opos = oconf.GetAtomPosition(i)
      if not lstEq(pos, opos):
        return 0
    im += 1
  return 1


def compareMatrices(bm1, bm2, map, tol=1.0e-5):
  N = numpy.shape(bm1)[0]
  for i in range(1, N):
    for j in range(i):
      l, m = map[i], map[j]
      if (l < m):
        l, m = m, l
      if (abs(bm1[l, m] - bm2[i, j]) > tol):
        return 0

      if (abs(bm1[m, l] - bm2[j, i]) > tol):
        return 0

  return 1


def compareOrder(smi1, smi2, tol=1.0e-5):
  m1 = Chem.MolFromSmiles(smi1)
  m2 = Chem.MolFromSmiles(smi2)
  bm1 = rdDistGeom.GetMoleculeBoundsMatrix(m1)
  bm2 = rdDistGeom.GetMoleculeBoundsMatrix(m2)
  map = m1.GetSubstructMatch(m2)
  return compareMatrices(bm1, bm2, map, tol)


def computeDist(lst1, lst2):
  res = 0.0
  for i, val in enumerate(lst1):
    res += (val - lst2[i]) * (val - lst2[i])
  res = math.sqrt(res)
  return res


def computeChiralVol(pt1, pt2, pt3, pt4):
  v1 = pt1 - pt4
  v2 = pt2 - pt4
  v3 = pt3 - pt4
  cp = v2.CrossProduct(v3)
  vol = v1.DotProduct(cp)
  return vol


class TestCase(unittest.TestCase):

  def setUp(self):
    pass

  def _test0Cdk2(self):
    fileN = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'DistGeomHelpers', 'test_data',
                         'cis_trans_cases.csv')

    ofile = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'DistGeomHelpers', 'test_data',
                         'embedDistOpti.sdf')
    self.assertTrue(compareWithOld(fileN, ofile))

  def test1Small(self):
    #writer = Chem.SDWriter("test.sdf")
    # single double and tripple atoms cases should not fail
    mol = Chem.MolFromSmiles('O')
    rdDistGeom.EmbedMolecule(mol, 10, 1)
    conf = mol.GetConformer()
    self.assertTrue(lstEq(conf.GetAtomPosition(0), [0.0, 0.0, 0.0]))
    # writer.write(mol)

    mol = Chem.MolFromSmiles('CO')
    rdDistGeom.EmbedMolecule(mol, 10, 1)
    conf = mol.GetConformer()
    self.assertTrue(lstEq(conf.GetAtomPosition(0), [0.69192, 0.0, 0.0]))
    self.assertTrue(lstEq(conf.GetAtomPosition(1), [-0.69192, 0.0, 0.0]))
    # writer.write(mol)

    mol = Chem.MolFromSmiles('CCC')
    rdDistGeom.EmbedMolecule(mol, 10, 1)
    conf = mol.GetConformer()
    self.assertTrue(lstEq(conf.GetAtomPosition(0), [-1.21676, -0.2989, 0.0]))
    self.assertTrue(lstEq(conf.GetAtomPosition(1), [-0.00604, 0.59337, 0.0]))
    self.assertTrue(lstEq(conf.GetAtomPosition(2), [1.22281, -0.29446, 0.0]))
    # writer.write(mol)

    mol = Chem.MolFromSmiles('O=C=O')
    rdDistGeom.EmbedMolecule(mol, 10, 1)
    conf = mol.GetConformer()

    # writer.write(mol)
    self.assertTrue(lstEq(conf.GetAtomPosition(0), [-1.2180, -0.06088, 0.0]))
    self.assertTrue(lstEq(conf.GetAtomPosition(1), [-0.00408, 0.12116, 0.0]))
    self.assertTrue(lstEq(conf.GetAtomPosition(2), [1.22207, -0.060276, 0.0]))

    mol = Chem.MolFromSmiles('C=C=C=C')
    rdDistGeom.EmbedMolecule(mol, 10, 1, useExpTorsionAnglePrefs=False, useBasicKnowledge=False)
    conf = mol.GetConformer()

    # writer.write(mol)

    d1 = computeDist(conf.GetAtomPosition(0), conf.GetAtomPosition(1))
    self.assertTrue(feq(d1, 1.31, 0.01))
    d2 = computeDist(conf.GetAtomPosition(0), conf.GetAtomPosition(2))
    self.assertTrue(feq(d2, 2.59, 0.05))
    d3 = computeDist(conf.GetAtomPosition(0), conf.GetAtomPosition(3))
    self.assertTrue(feq(d3, 3.84, 0.1))
    d4 = computeDist(conf.GetAtomPosition(1), conf.GetAtomPosition(2))
    self.assertTrue(feq(d4, 1.29, 0.01))
    d5 = computeDist(conf.GetAtomPosition(1), conf.GetAtomPosition(3))
    self.assertTrue(feq(d5, 2.54, 0.1))
    d6 = computeDist(conf.GetAtomPosition(2), conf.GetAtomPosition(3))
    self.assertTrue(feq(d6, 1.31, 0.01))

  def test2Utils(self):
    mol = Chem.MolFromSmiles('CC')
    bm = rdDistGeom.GetMoleculeBoundsMatrix(mol)
    self.assertTrue(bm[1, 0] > 0)
    self.assertTrue(bm[0, 1] > 0)
    self.assertTrue(bm[0, 1] >= bm[1, 0])
    self.assertTrue(bm[1, 0] < 1.510)
    self.assertTrue(bm[0, 1] > 1.510)

  def test3MultiConf(self):
    mol = Chem.MolFromSmiles("CC(C)(C)c(cc12)n[n]2C(=O)/C=C(N1)/COC")
    cids = rdDistGeom.EmbedMultipleConfs(mol, 10, maxAttempts=30, randomSeed=100,
                                         useExpTorsionAnglePrefs=False, useBasicKnowledge=False)
    energies = [
      116.330, 106.246, 109.816, 104.890, 93.060, 140.803, 139.253, 95.820, 123.591, 108.655
    ]
    nenergies = []
    for cid in cids:
      ff = ChemicalForceFields.UFFGetMoleculeForceField(mol, 10.0, cid)
      ee = ff.CalcEnergy()
      nenergies.append(ee)
    # print(['%.3f' % x for x in nenergies])
    # print(nenergies)
    self.assertTrue(lstEq(energies, nenergies, tol=1e-2))

  def test4OrderDependence(self):
    self.assertTrue(compareOrder("CC(C)(C)C(=O)NC(C1)CC(N2C)CCC12",
                                 "CN1C2CCC1CC(NC(=O)C(C)(C)C)C2"))
    # issue 230
    self.assertTrue(compareOrder("C#CC(C)(C)N(CN1)C\\N=C/1SC", "CSC1=NCN(C(C)(C)C#C)CN1"))
    # issue 232
    self.assertTrue(compareOrder("CC(C)(C)C(=O)NC(C1)CC(N2C)CCC12",
                                 "CN1C2CCC1CC(NC(=O)C(C)(C)C)C2"))

  def test5Issue285(self):
    m = Chem.MolFromSmiles('CNC=O')
    cs = rdDistGeom.EmbedMultipleConfs(m, 10)
    for i, ci in enumerate(cs):
      for j in range(i + 1, len(cs)):
        cj = cs[j]
        self.assertTrue(Chem.MolToMolBlock(m, confId=ci) != Chem.MolToMolBlock(m, confId=cj))

  def test6RmsPruning(self):
    smiles = [
      'CC(C)CC(NC(C1[N+]CCC1)=O)C([O-])=O', 'CC(NC(CO)C(O)c1ccc([N+]([O-])=O)cc1)=O',
      'CC([N+])C(NC(C)C(N1C(C=O)CCC1)=O)=O', 'CC(NC1C(O)C=C(C([O-])=O)OC1C(O)C(O)CO)=O',
      'CCCC=C(NC(C1CC1(C)C)=O)C([O-])=O', 'OCC(O)C(O)C(Cn1c2c(cc(C)c(C)c2)nc-2c(=O)[nH]c(=O)nc12)O'
    ]

    nconfs = []
    expected = [4, 5, 5, 4, 5, 4]
    expected = [3, 3, 5, 4, 4, 4]
    for smi in smiles:
      mol = Chem.MolFromSmiles(smi)
      cids = rdDistGeom.EmbedMultipleConfs(mol, 50, maxAttempts=30, randomSeed=100,
                                           pruneRmsThresh=1.5)
      nconfs.append(len(cids))

    d = [abs(x - y) for x, y in zip(expected, nconfs)]
    # print(nconfs)
    self.assertTrue(max(d) <= 1)

    # previous settings
    params = rdDistGeom.ETKDG()
    params.randomSeed = 100
    params.maxIterations = 30
    params.pruneRmsThresh = 1.5
    params.useSymmetryForPruning = False
    nconfs = []
    expected = [4, 5, 5, 4, 5, 4]
    for smi in smiles:
      mol = Chem.MolFromSmiles(smi)
      cids = rdDistGeom.EmbedMultipleConfs(mol, 50, params)
      nconfs.append(len(cids))

    d = [abs(x - y) for x, y in zip(expected, nconfs)]
    # print(nconfs)
    self.assertTrue(max(d) <= 1)

  def test6Chirality(self):
    # turn on chirality and we should get chiral volume that is pretty consistent and
    # positive
    tgtVol = 13.0
    smiles = "Cl[C@](C)(F)Br"
    mol = Chem.MolFromSmiles(smiles)
    cids = rdDistGeom.EmbedMultipleConfs(mol, 30, maxAttempts=30, randomSeed=100)
    self.assertTrue(len(cids) == 30)
    for cid in cids:
      conf = mol.GetConformer(cid)
      vol = computeChiralVol(conf.GetAtomPosition(0), conf.GetAtomPosition(2),
                             conf.GetAtomPosition(3), conf.GetAtomPosition(4))
      self.assertTrue(abs(vol - tgtVol) < 1)

    # turn of chirality and now we should see both chiral forms
    smiles = "ClC(C)(F)Br"
    mol = Chem.MolFromSmiles(smiles)
    cids = rdDistGeom.EmbedMultipleConfs(mol, 30, maxAttempts=30, randomSeed=120)
    self.assertTrue(len(cids) == 30)
    nPos = 0
    nNeg = 0
    for cid in cids:
      conf = mol.GetConformer(cid)
      vol = computeChiralVol(conf.GetAtomPosition(0), conf.GetAtomPosition(2),
                             conf.GetAtomPosition(3), conf.GetAtomPosition(4))
      self.assertTrue(abs(vol - tgtVol) < 1 or abs(vol + tgtVol) < 1)
      if vol < 0:
        nNeg += 1
      else:
        nPos += 1
    self.assertTrue(nPos > 0)
    self.assertTrue(nNeg > 0)

    tgtVol = 3.0
    for i in range(10):
      smiles = "Cl[C@H](F)Br"
      mol = Chem.MolFromSmiles(smiles)
      ci = rdDistGeom.EmbedMolecule(mol, 30, (i + 1) * 10)
      conf = mol.GetConformer(ci)
      vol = computeChiralVol(conf.GetAtomPosition(0), conf.GetAtomPosition(1),
                             conf.GetAtomPosition(2), conf.GetAtomPosition(3))
      self.assertTrue(abs(vol - tgtVol) < 1, "%s %s" % (vol, tgtVol))

    tgtVol = 3.5
    expected = [-3.62, -3.67, -3.72, 3.91, 3.95, 3.98, 3.90, 3.94, 3.98, 3.91]
    nPos = 0
    nNeg = 0
    for i in range(30):
      smiles = "ClC(F)Br"
      mol = Chem.MolFromSmiles(smiles)
      ci = rdDistGeom.EmbedMolecule(mol, 30, (i + 1) * 10)
      conf = mol.GetConformer(ci)
      vol = computeChiralVol(conf.GetAtomPosition(0), conf.GetAtomPosition(1),
                             conf.GetAtomPosition(2), conf.GetAtomPosition(3))
      self.assertTrue(abs(vol - tgtVol) < 1 or abs(vol + tgtVol) < 1)
      if vol < 0:
        nNeg += 1
      else:
        nPos += 1

    self.assertTrue(nPos > 0)
    self.assertTrue(nNeg > 0)

    smiles = "Cl[C@H](F)Br"
    m = Chem.MolFromSmiles(smiles)
    mol = Chem.AddHs(m)
    cids = rdDistGeom.EmbedMultipleConfs(mol, 10, maxAttempts=30, randomSeed=100)
    self.assertTrue(len(cids) == 10)
    tgtVol = 10.5
    for cid in cids:
      conf = mol.GetConformer(cid)
      vol = computeChiralVol(conf.GetAtomPosition(0), conf.GetAtomPosition(2),
                             conf.GetAtomPosition(3), conf.GetAtomPosition(4))
      self.assertTrue(abs(vol - tgtVol) < 2.)

    # let's try a little more complicated system
    expectedV1 = -2.0
    expectedV2 = -2.9

    for i in range(5):
      smi = "C1=CC=C(C=C1)[C@H](OC1=C[NH]N=C1)C(=O)[NH]C[C@H](Cl)C1=CC=NC=C1"
      mol = Chem.MolFromSmiles(smi)
      ci = rdDistGeom.EmbedMolecule(mol, randomSeed=(i + 1) * 15)
      self.assertTrue(ci >= 0)
      ff = ChemicalForceFields.UFFGetMoleculeForceField(mol, 10.0, ci)
      ff.Minimize()

      conf = mol.GetConformer(ci)
      vol1 = computeChiralVol(conf.GetAtomPosition(6), conf.GetAtomPosition(3),
                              conf.GetAtomPosition(7), conf.GetAtomPosition(13))
      self.assertTrue(abs(vol1 - expectedV1) < 1 or abs(vol1 + expectedV1) < 1)
      if vol1 < 0:
        nNeg += 1
      else:
        nPos += 1

      vol2 = computeChiralVol(conf.GetAtomPosition(17), conf.GetAtomPosition(16),
                              conf.GetAtomPosition(18), conf.GetAtomPosition(19))
      self.assertTrue(abs(vol2 - expectedV2) < 1 or abs(vol2 + expectedV2) < 1)

    # remove the chiral specification and we should see other chiral
    # forms of the compound
    expectedV1 = 2.0  # [-2.30, -2.31, -2.30,  2.30, -1.77]
    expectedV2 = 2.8  # [2.90,  2.89,  2.69, -2.90, -2.93]

    self.assertTrue(nPos > 0)
    self.assertTrue(nNeg > 0)
    for i in range(5):
      smi = "C1=CC=C(C=C1)C(OC1=C[NH]N=C1)C(=O)[NH]CC(Cl)C1=CC=NC=C1"
      mol = Chem.MolFromSmiles(smi)
      ci = rdDistGeom.EmbedMolecule(mol, 30, (i + 1) * 10)
      ff = ChemicalForceFields.UFFGetMoleculeForceField(mol, 10.0, ci)
      ff.Minimize()

      conf = mol.GetConformer(ci)
      vol1 = computeChiralVol(conf.GetAtomPosition(6), conf.GetAtomPosition(3),
                              conf.GetAtomPosition(7), conf.GetAtomPosition(13))
      vol2 = computeChiralVol(conf.GetAtomPosition(17), conf.GetAtomPosition(16),
                              conf.GetAtomPosition(18), conf.GetAtomPosition(19))
      self.assertTrue(abs(abs(vol1) - expectedV1) < 1.0)
      self.assertTrue(abs(abs(vol2) - expectedV2) < 1.0)

  def test7ConstrainedEmbedding(self):
    ofile = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'DistGeomHelpers', 'test_data',
                         'constrain1.sdf')
    suppl = Chem.SDMolSupplier(ofile)
    ref = next(suppl)
    probe = copy.deepcopy(ref)

    cMap = {}
    for i in range(5):
      cMap[i] = ref.GetConformer().GetAtomPosition(i)
    ci = rdDistGeom.EmbedMolecule(probe, coordMap=cMap, randomSeed=23)
    self.assertTrue(ci > -1)
    algMap = list(zip(range(5), range(5)))
    ssd = rdMolAlign.AlignMol(probe, ref, atomMap=algMap)
    self.assertTrue(ssd < 0.1)

  def test8MultiThreadMultiConf(self):
    if (rdBase.rdkitBuild.split('|')[2] != "MINGW"):
      ENERGY_TOLERANCE = 1.0e-6
      MSD_TOLERANCE = 1.0e-6
    else:
      ENERGY_TOLERANCE = 1.0
      MSD_TOLERANCE = 1.0e-5
    mol = Chem.AddHs(Chem.MolFromSmiles("CC(C)(C)c(cc12)n[n]2C(=O)/C=C(N1)/COC"))
    cids = rdDistGeom.EmbedMultipleConfs(mol, 200, maxAttempts=30, randomSeed=100)
    energies = []
    for cid in cids:
      ff = ChemicalForceFields.UFFGetMoleculeForceField(mol, 10.0, cid)
      ee = ff.CalcEnergy()
      energies.append(ee)

    mol2 = Chem.AddHs(Chem.MolFromSmiles("CC(C)(C)c(cc12)n[n]2C(=O)/C=C(N1)/COC"))
    cids2 = rdDistGeom.EmbedMultipleConfs(mol2, 200, maxAttempts=30, randomSeed=100, numThreads=4)
    self.assertTrue(lstEq(cids, cids2))
    nenergies = []
    for cid in cids2:
      ff = ChemicalForceFields.UFFGetMoleculeForceField(mol2, 10.0, cid)
      ee = ff.CalcEnergy()
      nenergies.append(ee)

    self.assertTrue(lstEq(energies, nenergies, tol=ENERGY_TOLERANCE))

    for cid in cids:
      msd = 0.0
      for i in range(mol.GetNumAtoms()):
        msd += (mol.GetConformer().GetAtomPosition(i) -
                mol2.GetConformer().GetAtomPosition(i)).LengthSq()
      msd /= mol.GetNumAtoms()
      self.assertTrue(msd < MSD_TOLERANCE)

  def _compareConfs(self, mol, ref, molConfId, refConfId):
    self.assertEqual(mol.GetNumAtoms(), ref.GetNumAtoms())
    molConf = mol.GetConformer(molConfId)
    refConf = ref.GetConformer(refConfId)
    for i in range(mol.GetNumAtoms()):
      mp = molConf.GetAtomPosition(i)
      rp = refConf.GetAtomPosition(i)
      self.assertAlmostEqual((mp - rp).Length(), 0.0, 3)

  def test9EmbedParams(self):
    mol = Chem.AddHs(Chem.MolFromSmiles('OCCC'))
    fn = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'DistGeomHelpers', 'test_data',
                      'simple_torsion.dg.mol')
    ref = Chem.MolFromMolFile(fn, removeHs=False)
    params = rdDistGeom.EmbedParameters()
    params.randomSeed = 42
    self.assertEqual(rdDistGeom.EmbedMolecule(mol, params), 0)
    self._compareConfs(mol, ref, 0, 0)

    fn = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'DistGeomHelpers', 'test_data',
                      'simple_torsion.etdg.mol')
    ref = Chem.MolFromMolFile(fn, removeHs=False)
    params = rdDistGeom.EmbedParameters()
    params.randomSeed = 42
    params.useExpTorsionAnglePrefs = True
    self.assertEqual(rdDistGeom.EmbedMolecule(mol, params), 0)
    self._compareConfs(mol, ref, 0, 0)
    params = rdDistGeom.ETDG()
    params.randomSeed = 42
    self.assertEqual(rdDistGeom.EmbedMolecule(mol, params), 0)
    self._compareConfs(mol, ref, 0, 0)

    fn = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'DistGeomHelpers', 'test_data',
                      'simple_torsion.etkdg.mol')
    ref = Chem.MolFromMolFile(fn, removeHs=False)
    params = rdDistGeom.EmbedParameters()
    params.randomSeed = 42
    params.useExpTorsionAnglePrefs = True
    params.useBasicKnowledge = True
    self.assertEqual(rdDistGeom.EmbedMolecule(mol, params), 0)
    self._compareConfs(mol, ref, 0, 0)
    params = rdDistGeom.ETKDG()
    params.randomSeed = 42
    self.assertEqual(rdDistGeom.EmbedMolecule(mol, params), 0)
    self._compareConfs(mol, ref, 0, 0)

    fn = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'DistGeomHelpers', 'test_data',
                      'simple_torsion.kdg.mol')
    ref = Chem.MolFromMolFile(fn, removeHs=False)
    params = rdDistGeom.EmbedParameters()
    params.randomSeed = 42
    params.useBasicKnowledge = True
    self.assertEqual(rdDistGeom.EmbedMolecule(mol, params), 0)
    self._compareConfs(mol, ref, 0, 0)
    params = rdDistGeom.KDG()
    params.randomSeed = 42
    self.assertEqual(rdDistGeom.EmbedMolecule(mol, params), 0)
    self._compareConfs(mol, ref, 0, 0)

  def test10ETKDGv2(self):
    mol = Chem.AddHs(Chem.MolFromSmiles('n1cccc(C)c1ON'))
    fn = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'DistGeomHelpers', 'test_data',
                      'torsion.etkdg.v2.mol')
    ref = Chem.MolFromMolFile(fn, removeHs=False)
    params = rdDistGeom.ETKDGv2()
    params.randomSeed = 42
    self.assertEqual(rdDistGeom.EmbedMolecule(mol, params), 0)
    self._compareConfs(mol, ref, 0, 0)

  def assertDeterministicWithSeed(self, seed):
    input_mol = Chem.MolFromSmiles('CN(Cc1cnc2nc(N)nc(N)c2n1)c1ccc(C(=O)NC(CCC(=O)O)C(=O)O)cc1')

    params = AllChem.ETKDG()
    params.pruneRmsThresh = -1.0  # skip internal RMSD pruning
    if seed is not None:
      params.randomSeed = seed

    firstMol = Chem.AddHs(input_mol)
    firstIds = AllChem.EmbedMultipleConfs(firstMol, 11, params)

    secondMol = Chem.AddHs(input_mol)
    secondIds = AllChem.EmbedMultipleConfs(secondMol, 11, params)

    self.assertEqual(list(firstIds), list(secondIds))
    self.assertEqual(firstMol.GetNumConformers(), secondMol.GetNumConformers())

    nonDeterministic = False
    for confIdx in range(firstMol.GetNumConformers()):

      firstConf = firstMol.GetConformer(confIdx)
      secondConf = secondMol.GetConformer(confIdx)

      firstPositions = firstConf.GetPositions()
      secondPositions = secondConf.GetPositions()

      d = firstPositions - secondPositions
      rmsd = numpy.sqrt(numpy.sum(d * d))
      if seed >= 0:
        self.assertEqual(rmsd, 0.0)
      elif rmsd != 0.0:
        nonDeterministic = True
    if seed < 0:
      self.assertTrue(nonDeterministic)

  def testETKDGIsDeterministic(self):
    self.assertDeterministicWithSeed(-1)  # not deterministic
    self.assertDeterministicWithSeed(0)  # deterministic
    self.assertDeterministicWithSeed(1)  # deterministic
    # as large as we can go without overflowing since 11 * 195225786 should not overflow the int
    self.assertDeterministicWithSeed(195225786)
    self.assertDeterministicWithSeed(195225787)  # one higher seed will overflow though
    # another large seeds that shouldn't overflow internals and make them non-deterministic
    self.assertDeterministicWithSeed(0x1CEB00DA)

  def testGithub1763(self):
    mol = Chem.MolFromSmiles('CCCCC')
    bm1 = rdDistGeom.GetMoleculeBoundsMatrix(mol)
    bm2 = rdDistGeom.GetMoleculeBoundsMatrix(mol, doTriangleSmoothing=False)
    self.assertTrue(bm1[0, 4] < bm2[0, 4])

  def testGithub2057(self):
    # ensure that ETKDG is the default Embedder
    mol = Chem.AddHs(Chem.MolFromSmiles('OCCC'))
    fn = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'DistGeomHelpers', 'test_data',
                      'simple_torsion.etkdg.mol')
    ref = Chem.MolFromMolFile(fn, removeHs=False)
    self.assertEqual(rdDistGeom.EmbedMolecule(mol, randomSeed=42), 0)
    self._compareConfs(mol, ref, 0, 0)

  def testProvidingBoundsMatrix(self):
    m1 = Chem.MolFromSmiles("C1CCC1C")
    bm1 = rdDistGeom.GetMoleculeBoundsMatrix(m1)
    bm1[0, 3] = 1.21
    bm1[3, 0] = 1.20
    bm1[2, 3] = 1.21
    bm1[3, 2] = 1.20
    bm1[4, 3] = 1.21
    bm1[3, 4] = 1.20
    DG.DoTriangleSmoothing(bm1)
    ps = rdDistGeom.EmbedParameters()
    ps.useRandomCoords = True
    ps.SetBoundsMat(bm1)
    ps.randomSeed = 0xf00d
    self.assertEqual(rdDistGeom.EmbedMolecule(m1, ps), 0)
    conf = m1.GetConformer()
    self.assertAlmostEqual((conf.GetAtomPosition(3) - conf.GetAtomPosition(0)).Length(), 1.2,
                           delta=0.05)
    self.assertAlmostEqual((conf.GetAtomPosition(3) - conf.GetAtomPosition(2)).Length(), 1.2,
                           delta=0.05)
    self.assertAlmostEqual((conf.GetAtomPosition(3) - conf.GetAtomPosition(4)).Length(), 1.2,
                           delta=0.05)

  def testProvidingCPCI(self):
    """
        test for a ring molecule, repeated generating a conformer with and without enforcing 
        an additional +ve interaction between a pair of non-bonded atoms (termed CPCI, 
        custom pairwise charge-like interaciton), in every iteration, applying CPCI should
        yield a conformer where this pair of atoms are further apart.
        """
    for i in range(5):
      ps = rdDistGeom.EmbedParameters()
      ps.randomSeed = i
      ps.useBasicKnowledge = True
      ps.useRandomCoords = False
      m1 = Chem.MolFromSmiles("C1CCCC1C")
      self.assertEqual(rdDistGeom.EmbedMolecule(m1, ps), 0)

      m2 = Chem.MolFromSmiles("C1CCCC1C")
      ps = rdDistGeom.EmbedParameters()
      ps.randomSeed = i
      ps.useRandomCoords = False
      ps.useBasicKnowledge = True
      ps.SetCPCI({(0, 3): 0.9})
      self.assertEqual(rdDistGeom.EmbedMolecule(m2, ps), 0)

      conf1 = m1.GetConformer()
      conf2 = m2.GetConformer()
      self.assertTrue((conf2.GetAtomPosition(3) - conf2.GetAtomPosition(0)).Length() > (
        conf1.GetAtomPosition(3) - conf1.GetAtomPosition(0)).Length())

  def testScaleBoundsMatForce(self):
    """
        for pentane, set a target distance for the 1-5 distance, and generate conformers with changing weights for (all) the atom pair distance restraints,
        the conformer with the stronger weight for the atom pairs will always have a 1-5 distance closer to the target value than that with the weaker weight.
        """
    target = 4
    for i in range(5):
      ps = rdDistGeom.EmbedParameters()
      ps.randomSeed = i
      ps.useBasicKnowledge = True
      ps.useRandomCoords = False
      m1 = Chem.MolFromSmiles("CCCCC")
      bm1 = rdDistGeom.GetMoleculeBoundsMatrix(m1)
      bm1[0, 4] = target
      bm1[4, 0] = target
      DG.DoTriangleSmoothing(bm1)
      ps.boundsMatForceScaling = 0.1
      ps.SetBoundsMat(bm1)
      self.assertEqual(rdDistGeom.EmbedMolecule(m1, ps), 0)

      m2 = Chem.MolFromSmiles("CCCCC")
      ps = rdDistGeom.EmbedParameters()
      ps.randomSeed = i
      ps.useBasicKnowledge = True
      ps.useRandomCoords = False
      ps.boundsMatForceScaling = 10
      ps.SetBoundsMat(bm1)
      self.assertEqual(rdDistGeom.EmbedMolecule(m2, ps), 0)

      conf1 = m1.GetConformer()
      conf2 = m2.GetConformer()
      self.assertTrue(
        abs((conf2.GetAtomPosition(4) - conf2.GetAtomPosition(0)).Length() -
            target) < abs((conf1.GetAtomPosition(4) - conf1.GetAtomPosition(0)).Length() - target))

  def testETKDGv3amide(self):
    """
        test for a macrocycle molecule, ETKDGv3 samples trans amide
    """

    def get_atom_mapping(mol, smirks="[O:1]=[C:2]@;-[NX3:3]-[H:4]"):
      qmol = Chem.MolFromSmarts(smirks)
      ind_map = {}
      for atom in qmol.GetAtoms():
        map_num = atom.GetAtomMapNum()
        if map_num:
          ind_map[map_num - 1] = atom.GetIdx()
      map_list = [ind_map[x] for x in sorted(ind_map)]
      matches = list()
      for match in mol.GetSubstructMatches(qmol, uniquify=False):
        mas = [match[x] for x in map_list]
        matches.append(tuple(mas))
      return matches

    smiles = "C1CCC(=O)NCCCCCC(=O)NC1"
    smiles_mol = Chem.MolFromSmiles(smiles)
    mol = Chem.AddHs(smiles_mol)
    params = AllChem.ETKDGv3()
    params.seed = 42
    AllChem.EmbedMolecule(mol, params)
    conf = mol.GetConformer(0)
    for torsion in get_atom_mapping(mol):
      a1, a2, a3, a4 = [conf.GetAtomPosition(i) for i in torsion]
      self.assertAlmostEqual(abs(ComputeSignedDihedralAngle(a1, a2, a3, a4)), 3.14, delta=0.1)

  def testGetTorsionBonds(self):
    m = Chem.AddHs(Chem.MolFromSmiles('CCCC'))
    ts = rdDistGeom.GetExperimentalTorsions(m)
    self.assertEqual(len(ts), 1)
    self.assertEqual(ts[0]["bondIndex"], 1)
    self.assertEqual(ts[0]["torsionIndex"], 229)
    self.assertEqual(ts[0]["smarts"], '[!#1:1][CX4H2:2]!@;-[CX4H2:3][!#1:4]')
    self.assertEqual(list(ts[0]["V"]), [0.0, 0.0, 4.0, 0.0, 0.0, 0.0])
    self.assertEqual(list(ts[0]["signs"]), [1, 1, 1, 1, 1, 1])
    self.assertEqual(list(ts[0]["atomIndices"]), [0, 1, 2, 3])
    params = rdDistGeom.ETKDGv3()
    ts = rdDistGeom.GetExperimentalTorsions(m, params)
    self.assertEqual(len(ts), 1)
    self.assertEqual(ts[0]["bondIndex"], 1)
    self.assertEqual(ts[0]["smarts"], '[!#1:1][CX4H2:2]!@;-[CX4H2:3][!#1:4]')
    self.assertEqual(ts[0]["torsionIndex"], 229)
    self.assertEqual(list(ts[0]["V"]), [0.0, 0.0, 4.0, 0.0, 0.0, 0.0])
    self.assertEqual(list(ts[0]["signs"]), [1, 1, 1, 1, 1, 1])
    self.assertEqual(list(ts[0]["atomIndices"]), [0, 1, 2, 3])


def testTrackFailures(self):
  params = AllChem.ETKDGv3()
  params.trackFailures = True
  params.maxIterations = 50
  params.randomSeed = 42
  mol = Chem.MolFromSmiles('C=CC1=C(N)Oc2cc1c(-c1cc(C(C)O)cc(=O)cc1C1NCC(=O)N1)c(OC)c2OC')
  mol = Chem.AddHs(mol)
  AllChem.EmbedMolecule(mol, params)
  cnts = params.GetFailureCounts()
  self.assertGreater(cnts[AllChem.EmbedFailureCauses.INITIAL_COORDS], 5)
  self.assertGreater(cnts[AllChem.EmbedFailureCauses.ETK_MINIMIZATION], 10)


if __name__ == '__main__':
  unittest.main()



#
#  Copyright (C) 2003-2017  Rational Discovery LLC
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
""" unit testing code for molecule suppliers

"""
import os
import tempfile
import unittest

from rdkit import Chem, RDConfig, RDLogger


class TestCase(unittest.TestCase):

  def tearDown(self):
    RDLogger.EnableLog('rdApp.error')

  def test1SDSupplier(self):
    fileN = os.path.join(RDConfig.RDCodeDir, 'VLib', 'NodeLib', 'test_data', 'NCI_aids.10.sdf')

    suppl = Chem.SDMolSupplier(fileN)
    ms = [x for x in suppl]
    self.assertEqual(len(ms), 10)

    # test repeating:
    ms = [x for x in suppl]
    self.assertEqual(len(ms), 10)

    # test reset:
    suppl.reset()
    m = next(suppl)
    self.assertEqual(m.GetProp('_Name'), '48')
    self.assertEqual(m.GetProp('NSC'), '48')
    self.assertEqual(m.GetProp('CAS_RN'), '15716-70-8')
    m = next(suppl)
    self.assertEqual(m.GetProp('_Name'), '78')
    self.assertEqual(m.GetProp('NSC'), '78')
    self.assertEqual(m.GetProp('CAS_RN'), '6290-84-2')

    suppl.reset()
    for _ in range(10):
      m = next(suppl)

    with self.assertRaises(StopIteration):
      m = next(suppl)

  def test2SmilesSupplier(self):
    fileN = os.path.join(RDConfig.RDCodeDir, 'VLib', 'NodeLib', 'test_data', 'pgp_20.txt')

    suppl = Chem.SmilesMolSupplier(fileN, delimiter='\t', smilesColumn=2, nameColumn=1, titleLine=1)
    ms = [x for x in suppl]
    self.assertEqual(len(ms), 20)

    # test repeating:
    ms = [x for x in suppl]
    self.assertEqual(len(ms), 20)
    # test reset:
    suppl.reset()
    m = next(suppl)
    self.assertEqual(m.GetProp('_Name'), 'ALDOSTERONE')
    self.assertEqual(m.GetProp('ID'), 'RD-PGP-0001')
    m = next(suppl)
    self.assertEqual(m.GetProp('_Name'), 'AMIODARONE')
    self.assertEqual(m.GetProp('ID'), 'RD-PGP-0002')
    suppl.reset()
    for _ in range(20):
      m = next(suppl)
    with self.assertRaises(StopIteration):
      m = next(suppl)

  def test3SmilesSupplier(self):
    txt = """C1CC1,1
CC(=O)O,3
fail,4
CCOC,5
"""
    RDLogger.DisableLog('rdApp.error')

    try:
      with tempfile.NamedTemporaryFile('w+', suffix='.csv', delete=False) as tmp:
        tmp.write(txt)
      suppl = Chem.SmilesMolSupplier(tmp.name, delimiter=',', smilesColumn=0, nameColumn=1,
                                     titleLine=0)
      ms = [x for x in suppl]
      suppl = None
      while ms.count(None):
        ms.remove(None)
      self.assertEqual(len(ms), 3)
    finally:
      os.unlink(tmp.name)


if __name__ == '__main__':
  unittest.main()



import time

from rdkit import Chem, rdBase
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors as rdMD
from rdkit.Chem.EState import AtomTypes, EStateIndices

print rdBase.rdkitVersion
print rdBase.boostVersion


def getEState(mol):

  return EStateIndices(mol)


def localopt(mol, steps=500):
  if mol.GetNumConformers() == 0:
    mol = make3D(mol)
  AllChem.MMFFOptimizeMolecule(mol, maxIters=steps)
  return mol


def make3D(mol, steps=50):
  mol = Chem.AddHs(mol)
  success = AllChem.EmbedMolecule(mol)
  if success == -1:  # Failed
    success = AllChem.EmbedMolecule(mol, useRandomCoords=True)
    if success == -1:
      raise Error, "Embedding failed!"
  mol = localopt(mol, steps)
  return mol


def get3D(m, is3d):
  if not is3d:
    m = Chem.AddHs(m)
    AllChem.EmbedMolecule(m)
    AllChem.MMFFOptimizeMolecule(m)
  r = rdMD.CalcAUTOCORR3D(m) + rdMD.CalcRDF(m) + rdMD.CalcMORSE(m) + rdMD.CalcWHIM(
    m) + rdMD.CalcGETAWAY(m)
  return r


def generateALL():
  m = Chem.MolFromSmiles('Cc1ccccc1')
  thefile = open('testAC.txt', 'w')
  filename = "/Users/mbp/Github/rdkit_mine/Code/GraphMol/Descriptors/test_data/PBF_egfr.sdf"
  suppl = Chem.SDMolSupplier(filename, removeHs=False)
  mols = [x for x in suppl]
  start = time.time()
  for m in mols:
    r = get3D(m, True)
    for item in r:
      thefile.write("%.3f," % item)
    thefile.write("\n")

  end = time.time()
  print end - start


thefile = open('testSMWHIM.txt', 'w')
writer = Chem.SDWriter('3Dsmallmol.sdf')
A = [
  '[H][H]', 'B', 'O=O', 'C', 'CC', 'CCC', 'CCCC', 'CCCCC', 'CCCCCC', 'CO', 'CCO', 'CCCO', 'CCCCO',
  'CCCCCO', 'CCCCCCO', 'CCl', 'CCCl', 'CCCCl', 'CCCCCl', 'CCCCCCl', 'CCCCCCCl', 'CBr', 'CCBr',
  'CCCBr', 'CCCCBr', 'CCCCCBr', 'CCCCCCBr', 'CI', 'CCI', 'CCCI', 'CCCCI', 'CCCCCI', 'CCCCCCI', 'CF',
  'CCF', 'CCCF', 'CCCCF', 'CCCCCF', 'CCCCCCF', 'CS', 'CCS', 'CCCS', 'CCCCS', 'CCCCCS', 'CCCCCCS',
  'CN', 'CCN', 'CCCN', 'CCCCN', 'CCCCCN', 'CCCCCCN'
]
for smi in A:
  m = Chem.MolFromSmiles(smi)
  m = localopt(m, 100)
  #r=get3D(m,True)
  print smi
  print "---------"
  r = rdMD.CalcWHIM(m)
  print "Ei:" + str(r[0]) + "," + str(r[1]) + "," + str(r[2]) + "\n"
  print "Gi:" + str(r[5]) + "," + str(r[6]) + "," + str(r[7]) + "\n"
  print "SI:" + str(rdMD.CalcSpherocityIndex(m))
  print "AS:" + str(rdMD.CalcAsphericity(m))
  print "EX:" + str(rdMD.CalcEccentricity(m))
  for item in r:
    thefile.write("%.3f," % item)
  thefile.write("\n")
  #m.SetProp("smi", smi)
  #writer.write(m)

thefile = open('testBPA.txt', 'w')
writer = Chem.SDWriter('3DBPAmol.sdf')
B = [
  'CN(C)CC(Br)c1ccccc1', 'CN(C)CC(Br)c1ccc(F)cc1', 'CN(C)CC(Br)c1ccc(Cl)cc1',
  'CN(C)CC(Br)c1ccc(Cl)cc1', 'CN(C)CC(Br)c1ccc(I)cc1', 'CN(C)CC(Br)c1ccc(C)cc1',
  'CN(C)CC(Br)c1cccc(F)c1', 'CN(C)CC(Br)c1cccc(Cl)c1', 'CN(C)CC(Br)c1cccc(Br)c1',
  'CN(C)CC(Br)c1cccc(I)c1', 'CN(C)CC(Br)c1cccc(C)c1', 'CN(C)CC(Br)c1ccc(F)c(Cl)c1',
  'CN(C)CC(Br)c1ccc(F)c(Br)c1', 'CN(C)CC(Br)c1ccc(F)c(C)c1', 'CN(C)CC(Br)c1ccc(Cl)c(Cl)c1',
  'CN(C)CC(Br)c1ccc(Cl)c(Br)c1', 'CN(C)CC(Br)c1ccc(Cl)c(C)c1', 'CN(C)CC(Br)c1ccc(Br)c(Cl)c1',
  'CN(C)CC(Br)c1ccc(Br)c(Br)c1', 'CN(C)CC(Br)c1ccc(Br)c(C)c1', 'CN(C)CC(Br)c1ccc(C)c(C)c1',
  'CN(C)CC(Br)c1ccc(C)c(Br)c1'
]
for smi in B:
  m = Chem.MolFromSmiles(smi)
  m = localopt(m, 100)
  #r=get3D(m,True)
  r = rdMD.CalcWHIM(m)
  for item in r:
    thefile.write("%.3f," % item)
  thefile.write("\n")
  #m.SetProp("smi", smi)
  #writer.write(m)

A = "G1w,G2w,G3w,Gw"
print dir(rdMD)



# $Id$
#
#  Copyright (C) 2003-2006  greg Landrum and Rational Discovery LLC
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
"""unit testing code for the DbResultSet object

"""
import unittest

from rdkit import RDConfig
from rdkit.Dbase.DbConnection import DbConnect
from rdkit.Dbase.DbResultSet import DbResultSet, RandomAccessDbResultSet


class TestCase(unittest.TestCase):

  def setUp(self):
    self.dbName = RDConfig.RDTestDatabase
    self.conn = DbConnect(self.dbName)
    self.curs = self.conn.GetCursor()

  def test1(self):
    """ test indexing in, ensure acceptable error conditions """
    cmd = 'select * from ten_elements'
    resultSet = RandomAccessDbResultSet(self.curs, self.conn, cmd)
    self.assertRaises(IndexError, resultSet.__getitem__, -1)
    for i in range(12):
      try:
        _ = resultSet[i]
      except IndexError:
        assert i >= 10
    self.assertEqual(resultSet.GetColumnNames(), ('id', 'val'))
    self.assertEqual(resultSet.GetColumnTypes(), ('integer', 'integer'))
    self.assertEqual(resultSet.GetColumnNamesAndTypes(), (('id', 'integer'), ('val', 'integer')))

    cmd = 'select * from ten_elements_dups'
    resultSet = RandomAccessDbResultSet(self.curs, self.conn, cmd)
    for i in range(22):
      try:
        _ = resultSet[i]
      except IndexError:
        assert i >= 20

    cmd = 'select * from ten_elements_dups'
    resultSet = RandomAccessDbResultSet(self.curs, self.conn, cmd, removeDups=0)
    for i in range(22):
      try:
        _ = resultSet[i]
      except IndexError:
        assert i >= 10

    # Test iterator
    resultSet = RandomAccessDbResultSet(self.curs, self.conn, cmd, removeDups=0)
    self.assertEqual(next(resultSet), resultSet[0])
    self.assertEqual(len(list(resultSet)), 10)

  def test2(self):
    cmd = 'select * from ten_elements'
    resultSet = RandomAccessDbResultSet(self.curs, self.conn, cmd)
    assert len(resultSet) == 10
    for i in range(len(resultSet)):
      _ = resultSet[i]

  def test3(self):
    cmd = 'select * from ten_elements'
    resultSet = DbResultSet(self.curs, self.conn, cmd)
    r = [obj for obj in resultSet]
    self.assertEqual(len(r), 10)

    # Test iterator
    resultSet = DbResultSet(self.curs, self.conn, cmd)
    self.assertEqual(next(resultSet), (0, 11))
    self.assertEqual(len(list(resultSet)), 10)

  def test4(self):
    cmd = 'select * from ten_elements_dups'
    resultSet = DbResultSet(self.curs, self.conn, cmd)
    r = [obj for obj in resultSet]
    self.assertEqual(len(r), 20)

    resultSet = DbResultSet(self.curs, self.conn, cmd, removeDups=0)
    r = [obj for obj in resultSet]
    self.assertEqual(len(r), 10)

  def test5(self):
    cmd = 'select * from ten_elements_dups'
    resultSet = RandomAccessDbResultSet(self.curs, self.conn, cmd)
    self.assertEqual(len(resultSet), 20)
    r = [obj for obj in resultSet]
    self.assertEqual(len(r), 20)

    resultSet = RandomAccessDbResultSet(self.curs, self.conn, cmd)
    r = [obj for obj in resultSet]
    self.assertEqual(len(r), 20)
    self.assertEqual(len(resultSet), 20)

    resultSet = RandomAccessDbResultSet(self.curs, self.conn, cmd, removeDups=0)
    self.assertEqual(len(resultSet), 10)
    r = [obj for obj in resultSet]
    self.assertEqual(len(r), 10)

  def test6(self):
    cmd = 'select * from ten_elements_dups'
    resultSet = DbResultSet(self.curs, self.conn, cmd, removeDups=0)
    r = [obj for obj in resultSet]
    self.assertEqual(len(r), 10)


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



#
#  Copyright (C) 2000  greg Landrum
#
""" unit testing code for trees and decision trees (not learning/xvalidation) """

import copy
import os
import pickle
import unittest
from io import StringIO

from rdkit import RDConfig
from rdkit.ML.DecTree import Tree
from rdkit.TestRunner import redirect_stdout


class TreeTestCase(unittest.TestCase):

  def setUp(self):
    self.baseTree = Tree.TreeNode(None, 'root')
    self.pickleFileName = RDConfig.RDCodeDir + '/ML/DecTree/test_data/treeunit.pkl'

  def test_Tree(self):
    tree = Tree.TreeNode(None, 'root', label=0)
    self.assertEqual(tree.GetLevel(), 0)
    self.assertEqual(tree.GetName(), 'root')
    self.assertEqual(tree.GetData(), None)
    self.assertEqual(tree.GetTerminal(), False)
    self.assertEqual(tree.GetLabel(), 0)
    self.assertEqual(tree.GetParent(), None)
    self.assertEqual(tree.GetChildren(), [])

    for i in range(3):
      child = tree.AddChild('child {0}'.format(i), i + 1, data={'key': 'value'})
      self.assertEqual(child.GetLevel(), 1)
      self.assertEqual(child.GetName(), 'child {0}'.format(i))
      self.assertEqual(child.GetData(), {'key': 'value'})
      self.assertEqual(child.GetLabel(), i + 1)
      self.assertEqual(child.GetParent(), tree)
      self.assertEqual(child.GetChildren(), [])
    children = tree.GetChildren()
    self.assertEqual(len(children), 3)
    children[0].AddChild('terminal', 4, isTerminal=True)

    s = str(tree)
    self.assertIn('root', s)
    self.assertIn('    terminal', s)
    self.assertIn('  child 2', s)

    tree.NameTree(['a', 'b', 'c', 'd', 'e'])
    self.assertEqual(str(tree), 'a\n  b\n    terminal\n  c\n  d\n')

    tree.PruneChild(children[1])
    self.assertEqual(str(tree), 'a\n  b\n    terminal\n  d\n')

    f = StringIO()
    with redirect_stdout(f):
      tree.Print(showData=True)
    s = f.getvalue()
    self.assertIn('value', s)
    self.assertIn('None', s)

    f = StringIO()
    with redirect_stdout(f):
      tree.Print()
    s = f.getvalue()
    self.assertNotIn('value', s)
    self.assertNotIn('None', s)

    tree.Destroy()
    self.assertEqual(str(tree), 'a\n')

  def _readyTree(self):
    tree = self.baseTree
    tree.AddChild('child0')
    tree.AddChild('child1')

  def test5Equals(self):
    # " testing tree equals "
    nTree = Tree.TreeNode(None, 'root')
    self._readyTree()
    tTree = self.baseTree
    self.baseTree = nTree
    self._readyTree()
    assert tTree == self.baseTree, 'Equality test 1 failed. (bad Tree.__cmp__)'
    assert self.baseTree == tTree, 'Equality test 2 failed. (bad Tree.__cmp__)'
    tTree.AddChild('child2')
    assert tTree != self.baseTree, 'Inequality test 1 failed. (bad Tree.__cmp__)'
    assert self.baseTree != tTree, 'Inequality test 2 failed. (bad Tree.__cmp__)'

    self.assertTrue(tTree > self.baseTree, msg='Larger tree is greater')
    self.assertEqual(tTree.__cmp__(self.baseTree), 1)

  def test6PickleEquals(self):
    # " testing pickled tree equals "
    self._readyTree()
    pkl = pickle.dumps(self.baseTree)
    oTree = pickle.loads(pkl)

    assert oTree == self.baseTree, 'Pickle inequality test failed'
    self.assertEqual(oTree.__cmp__(self.baseTree), 0)

    self.baseTree.PruneChild(self.baseTree.GetChildren()[0])
    assert oTree != self.baseTree, 'Pickle inequality test failed (bad Tree.__cmp__)'
    self.assertEqual(abs(oTree.__cmp__(self.baseTree)), 1)

  def test7Copy(self):
    # " testing deepcopy on trees "
    self._readyTree()
    nTree = copy.deepcopy(self.baseTree)
    assert nTree == self.baseTree, 'deepcopy failed'

  def test8In(self):
    # " testing list membership "
    self._readyTree()
    nTree = copy.deepcopy(self.baseTree)
    nTree2 = copy.deepcopy(self.baseTree)
    nTree2.PruneChild(self.baseTree.GetChildren()[0])
    tList = [nTree2, nTree2, nTree]
    assert self.baseTree in tList, 'list membership (tree in list) failed'
    tList = [nTree2, nTree2]
    assert self.baseTree not in tList, 'list membership (tree not in list) failed'

  def test_exampleCode(self):
    try:
      f = StringIO()
      with redirect_stdout(f):
        Tree._exampleCode()
      self.assertTrue(os.path.isfile('save.pkl'))
      self.assertIn('tree==tree2 False', f.getvalue(), 'Example didn' 't run to end')
    finally:
      if os.path.isfile('save.pkl'):
        os.remove('save.pkl')


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



#  Copyright (C) 2001-2021  greg Landrum
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
"""basic unit testing code for the molecule boost wrapper

"""
import os
import pickle
import unittest

from rdkit import Chem, RDConfig


class TestCase(unittest.TestCase):

  def setUp(self):
    self.bigSmiList = [
      "CC1=CC(=O)C=CC1=O",
      "S(SC1=NC2=CC=CC=C2S1)C3=NC4=C(S3)C=CC=C4",
      "OC1=C(Cl)C=C(C=C1[N+]([O-])=O)[N+]([O-])=O",
      "[O-][N+](=O)C1=CNC(=N)S1",
      "NC1=CC2=C(C=C1)C(=O)C3=C(C=CC=C3)C2=O",
      "OC(=O)C1=C(C=CC=C1)C2=C3C=CC(=O)C(=C3OC4=C2C=CC(=C4Br)O)Br",
      "CN(C)C1=C(Cl)C(=O)C2=C(C=CC=C2)C1=O",
      "CC1=C(C2=C(C=C1)C(=O)C3=CC=CC=C3C2=O)[N+]([O-])=O",
      "CC(=NO)C(C)=NO",
      "C1=CC=C(C=C1)P(C2=CC=CC=C2)C3=CC=CC=C3",
      "CC(C)(C)C1=C(O)C=C(C(=C1)O)C(C)(C)C",
      "CC1=NN(C(=O)C1)C2=CC=CC=C2",
      "NC1=CC=NC2=C1C=CC(=C2)Cl",
      "CCCCCC[CH]1CCCCN1",
      "O=CC1=C2C=CC=CC2=CC3=C1C=CC=C3",
      "BrN1C(=O)CCC1=O",
      "CCCCCCCCCCCCCCCC1=C(N)C=CC(=C1)O",
      "C(COC1=C(C=CC=C1)C2=CC=CC=C2)OC3=CC=CC=C3C4=CC=CC=C4",
      "CCCCSCC",
      "CC(=O)NC1=NC2=C(C=C1)C(=CC=N2)O",
      "CC1=C2C=CC(=NC2=NC(=C1)O)N",
      "CCOC(=O)C1=CN=C2N=C(N)C=CC2=C1O",
      "CC1=CC(=NC=C1)N=CC2=CC=CC=C2",
      "C[N+](C)(C)CC1=CC=CC=C1",
      "C[N+](C)(C)C(=O)C1=CC=CC=C1",
      "ICCC(C1=CC=CC=C1)(C2=CC=CC=C2)C3=CC=CC=C3",
      "CC1=CC(=C(C[N+](C)(C)C)C(=C1)C)C",
      "C[C](O)(CC(O)=O)C1=CC=C(C=C1)[N+]([O-])=O",
      "CC1=CC=C(C=C1)C(=O)C2=CC=C(Cl)C=C2",
      "ON=CC1=CC=C(O)C=C1",
      "CC1=CC(=C(N)C(=C1)C)C",
      "CC1=CC=C(C=C1)C(=O)C2=CC=C(C=C2)[N+]([O-])=O",
      "CC(O)(C1=CC=CC=C1)C2=CC=CC=C2",
      "ON=CC1=CC(=CC=C1)[N+]([O-])=O",
      "OC1=C2C=CC(=CC2=NC=C1[N+]([O-])=O)Cl",
      "CC1=CC=CC2=NC=C(C)C(=C12)Cl",
      "CCC(CC)([CH](OC(N)=O)C1=CC=CC=C1)C2=CC=CC=C2",
      "ON=C(CC1=CC=CC=C1)[CH](C#N)C2=CC=CC=C2",
      "O[CH](CC1=CC=CC=C1)C2=CC=CC=C2",
      "COC1=CC=C(CC2=CC=C(OC)C=C2)C=C1",
      "CN(C)[CH](C1=CC=CC=C1)C2=C(C)C=CC=C2",
      "COC1=CC(=C(N)C(=C1)[N+]([O-])=O)[N+]([O-])=O",
      "NN=C(C1=CC=CC=C1)C2=CC=CC=C2",
      "COC1=CC=C(C=C1)C=NO",
      "C1=CC=C(C=C1)C(N=C(C2=CC=CC=C2)C3=CC=CC=C3)C4=CC=CC=C4",
      "C1=CC=C(C=C1)N=C(C2=CC=CC=C2)C3=CC=CC=C3",
      "CC1=C(C2=CC=CC=C2)C(=C3C=CC=CC3=N1)O",
      "CCC1=[O+][Cu]2([O+]=C(CC)C1)[O+]=C(CC)CC(=[O+]2)CC",
      "OC(=O)[CH](CC1=CC=CC=C1)C2=CC=CC=C2",
      "CCC1=C(N)C=C(C)N=C1",
    ]

  def _testPkl10(self):
    " testing 5k molecule pickles "
    inLines = open('%s/NCI/first_5K.smi' % (RDConfig.RDDataDir), 'r').readlines()
    smis = []
    for line in inLines:
      smis.append(line.split('\t')[0])
    for smi in smis:
      m = Chem.MolFromSmiles(smi)
      newM1 = pickle.loads(pickle.dumps(m))
      newSmi1 = Chem.MolToSmiles(newM1)
      newM2 = pickle.loads(pickle.dumps(newM1))
      newSmi2 = Chem.MolToSmiles(newM2)
      assert newM1.GetNumAtoms() == m.GetNumAtoms(), 'num atoms comparison failed'
      assert newM2.GetNumAtoms() == m.GetNumAtoms(), 'num atoms comparison failed'
      assert len(newSmi1) > 0, 'empty smi1'
      assert len(newSmi2) > 0, 'empty smi2'
      assert newSmi1 == newSmi2, 'string compare failed:\n%s\n\t!=\n%s\norig smiles:\n%s' % (
        newSmi1, newSmi2, smi)

  def testPkl1(self):
    " testing single molecule pickle "
    m = Chem.MolFromSmiles('CCOC')
    outS = Chem.MolToSmiles(m)
    m2 = pickle.loads(pickle.dumps(m))
    outS2 = Chem.MolToSmiles(m2)
    assert outS == outS2, "bad pickle: %s != %s" % (outS, outS2)

  def testPkl2(self):
    """ further pickle tests """
    smis = self.bigSmiList
    for smi in smis:
      m = Chem.MolFromSmiles(smi)
      newM1 = pickle.loads(pickle.dumps(m))
      newM2 = pickle.loads(pickle.dumps(newM1))
      oldSmi = Chem.MolToSmiles(newM1)
      newSmi = Chem.MolToSmiles(newM2)
      assert newM1.GetNumAtoms() == m.GetNumAtoms(), 'num atoms comparison failed'
      assert newM2.GetNumAtoms() == m.GetNumAtoms(), 'num atoms comparison failed'
      assert oldSmi == newSmi, 'string compare failed: %s != %s' % (oldSmi, newSmi)

  def testPkl(self):
    " testing molecule pickle "
    import tempfile
    f, self.fName = tempfile.mkstemp('.pkl')
    f = None
    self.m = Chem.MolFromSmiles('CC(=O)CC')
    outF = open(self.fName, 'wb+')
    pickle.dump(self.m, outF)
    outF.close()
    inF = open(self.fName, 'rb')
    m2 = pickle.load(inF)
    inF.close()
    try:
      os.unlink(self.fName)
    except Exception:
      pass
    oldSmi = Chem.MolToSmiles(self.m)
    newSmi = Chem.MolToSmiles(m2)
    assert oldSmi == newSmi, 'string compare failed'

  def testRings(self):
    " testing SSSR handling "
    m = Chem.MolFromSmiles('OC1C(O)C2C1C(O)C2O')
    for i in range(m.GetNumAtoms()):
      at = m.GetAtomWithIdx(i)
      n = at.GetAtomicNum()
      if n == 8:
        assert not at.IsInRingSize(4), 'atom %d improperly in ring' % (i)
      else:
        assert at.IsInRingSize(4), 'atom %d not in ring of size 4' % (i)

  @unittest.skipIf(not hasattr(Chem, "MolToJSON"), "MolInterchange support not enabled")
  def testJSON1(self):
    """ JSON test1 """
    for smi in self.bigSmiList:
      m = Chem.MolFromSmiles(smi)
      json = Chem.MolToJSON(m)
      nm = Chem.JSONToMols(json)[0]
      self.assertEqual(Chem.MolToSmiles(m), Chem.MolToSmiles(nm))

  @unittest.skipIf(not hasattr(Chem, "MolToJSON"), "MolInterchange support not enabled")
  def testJSON2(self):
    """ JSON test2 """
    ms = [Chem.MolFromSmiles(smi) for smi in self.bigSmiList]
    json = Chem.MolsToJSON(ms)
    nms = Chem.JSONToMols(json)
    #for nm in nms:
    #  Chem.SanitizeMol(nm)
    self.assertEqual(len(ms), len(nms))
    smis1 = [Chem.MolToSmiles(x) for x in ms]
    smis2 = [Chem.MolToSmiles(x) for x in nms]
    for i, (smi1, smi2) in enumerate(zip(smis1, smis2)):
      if smi1 != smi2:
        print(self.bigSmiList[i])
        print(smi1)
        print(smi2)
        print("-------")
    self.assertEqual(smis1, smis2)

  def testGithub4144(self):
    """ github4144: EnumerateStereiosomers clearing ring info """
    from rdkit.Chem import EnumerateStereoisomers
    m = Chem.MolFromSmiles('CSCc1cnc(C=Nn2c(C)nc3sc4c(c3c2=O)CCCCC4)s1')
    sssr = [list(x) for x in Chem.GetSymmSSSR(m)]
    sms = EnumerateStereoisomers.EnumerateStereoisomers(m)
    for sm in sms:
      sssr2 = [list(x) for x in Chem.GetSymmSSSR(sm)]
      self.assertEqual(sssr, sssr2)


if __name__ == '__main__':
  unittest.main()



#
#  Copyright (C) 2003-2006 Rational Discovery LLC
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
import io
import os
import pickle
import unittest

from rdkit import Chem, RDConfig
from rdkit.Chem import BuildFragmentCatalog, FragmentCatalog


def feq(n1, n2, tol=1e-4):
  return abs(n1 - n2) < tol


class TestCase(unittest.TestCase):

  def setUp(self):
    self.smiList = [
      "S(SC1=NC2=CC=CC=C2S1)C3=NC4=C(S3)C=CC=C4", "CC1=CC(=O)C=CC1=O",
      "OC1=C(Cl)C=C(C=C1[N+]([O-])=O)[N+]([O-])=O", "[O-][N+](=O)C1=CNC(=N)S1",
      "NC1=CC2=C(C=C1)C(=O)C3=C(C=CC=C3)C2=O",
      "OC(=O)C1=C(C=CC=C1)C2=C3C=CC(=O)C(=C3OC4=C2C=CC(=C4Br)O)Br",
      "CN(C)C1=C(Cl)C(=O)C2=C(C=CC=C2)C1=O", "CC1=C(C2=C(C=C1)C(=O)C3=CC=CC=C3C2=O)[N+]([O-])=O",
      "CC(=NO)C(C)=NO"
    ]
    self.smiList2 = [
      'OCCC',
      'CCC',
      'C=CC',
      'OC=CC',
      'CC(O)C',
      'C=C(O)C',
      'OCCCC',
      'CC(O)CC',
      'C=CCC',
      'CC=CC',
      'OC=CCC',
      'CC=C(O)C',
      'OCC=CC',
      'C=C(O)CC',
      'C=CC(O)C',
      'C=CCCO',
    ]
    self.list2Acts = [1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1]
    self.list2Obls = [(0, 1, 2), (1, 3), (1, 4, 5), (1, 6, 7), (0, 8), (0, 6, 9), (0, 1, 2, 3, 10),
                      (0, 1, 2, 8, 11), (1, 3, 4, 5, 12), (1, 4, 5, 13), (1, 3, 6, 7, 14),
                      (0, 1, 6, 7, 9, 15)]

    ffile = os.path.join(RDConfig.RDDataDir, 'FunctionalGroups.txt')
    self.catParams = FragmentCatalog.FragCatParams(1, 6, ffile)
    self.fragCat = FragmentCatalog.FragCatalog(self.catParams)
    self.fgen = FragmentCatalog.FragCatGenerator()

  def _fillCat(self, smilList):
    for smi in self.smiList2:
      mol = Chem.MolFromSmiles(smi)
      self.fgen.AddFragsFromMol(mol, self.fragCat)

  def _testBits(self, fragCat):
    fpgen = FragmentCatalog.FragFPGenerator()
    obits = [3, 2, 3, 3, 2, 3, 5, 5, 5, 4, 5, 6]
    obls = self.list2Obls
    suppl = Chem.SmilesMolSupplierFromText('\n'.join(self.smiList2), ',', 0, -1, 0)
    i = 0
    for mol in suppl:
      fp = fpgen.GetFPForMol(mol, fragCat)
      if i < len(obits):
        smi = Chem.MolToSmiles(mol)
        assert fp.GetNumOnBits() == obits[i], '%s: %s' % (smi, str(fp.GetOnBits()))
      obl = fp.GetOnBits()
      if i < len(obls):
        assert tuple(obl) == obls[i], '%s: %s' % (smi, obl)
      i += 1

  def test1CatGen(self):
    self._fillCat(self.smiList2)
    assert self.fragCat.GetNumEntries() == 21
    assert self.fragCat.GetFPLength() == 21
    self._testBits(self.fragCat)

  def test2CatStringPickle(self):
    self._fillCat(self.smiList2)

    # test non-binary pickle:
    cat2 = pickle.loads(pickle.dumps(self.fragCat))
    assert cat2.GetNumEntries() == 21
    assert cat2.GetFPLength() == 21
    self._testBits(cat2)

    # test binary pickle:
    cat2 = pickle.loads(pickle.dumps(self.fragCat, 1))
    assert cat2.GetNumEntries() == 21
    assert cat2.GetFPLength() == 21
    self._testBits(cat2)

  def test3CatFilePickle(self):
    with open(os.path.join(RDConfig.RDCodeDir, 'Chem', 'test_data', 'simple_catalog.pkl'),
              'r') as pklTFile:
      buf = pklTFile.read().replace('\r\n', '\n').encode('utf-8')
      pklTFile.close()
    with io.BytesIO(buf) as pklFile:
      cat = pickle.load(pklFile, encoding='bytes')
    assert cat.GetNumEntries() == 21
    assert cat.GetFPLength() == 21
    self._testBits(cat)

  def test4CatGuts(self):
    self._fillCat(self.smiList2)
    assert self.fragCat.GetNumEntries() == 21
    assert self.fragCat.GetFPLength() == 21
    #
    # FIX: (Issue 162)
    # bits like 11 and 15 are questionable here because the underlying
    #  fragments are symmetrical, so they can generate one of two
    #  text representations (i.e. there is nothing to distinguish
    #  between 'CC<-O>CC' and 'CCC<-O>C').
    # This ought to eventually be cleaned up.
    descrs = [
      (0, 'C<-O>C', 1, (34, )),
      (1, 'CC', 1, ()),
      (2, 'C<-O>CC', 2, (34, )),
      (3, 'CCC', 2, ()),
      (4, 'C=C', 1, ()),
      (5, 'C=CC', 2, ()),
      (6, 'C<-O>=C', 1, (34, )),
      (7, 'C<-O>=CC', 2, (34, )),
      (8, 'CC<-O>C', 2, (34, )),
      (9, 'C=C<-O>C', 2, (34, )),
      (10, 'C<-O>CCC', 3, (34, )),
      (11, 'CC<-O>CC', 3, (34, )),
      (12, 'C=CCC', 3, ()),
      (13, 'CC=CC', 3, ()),
      (14, 'C<-O>=CCC', 3, (34, )),
      (15, 'CC=C<-O>C', 3, (34, )),
      (16, 'C=CC<-O>', 2, (34, )),
    ]
    for i in range(len(descrs)):
      ID, d, order, ids = descrs[i]
      descr = self.fragCat.GetBitDescription(ID)
      assert descr == d, '%d: %s != %s' % (ID, descr, d)
      assert self.fragCat.GetBitOrder(ID) == order
      assert tuple(self.fragCat.GetBitFuncGroupIds(
        ID)) == ids, '%d: %s != %s' % (ID, str(self.fragCat.GetBitFuncGroupIds(ID)), str(ids))

  def _test5MoreComplex(self):
    lastIdx = 0
    ranges = {}
    suppl = Chem.SmilesMolSupplierFromText('\n'.join(self.smiList), ',', 0, -1, 0)
    for i, mol in enumerate(suppl):
      nEnt = self.fgen.AddFragsFromMol(mol, self.fragCat)
      ranges[i] = range(lastIdx, lastIdx + nEnt)
      lastIdx += nEnt
    # now make sure that those bits are contained in the signatures:
    fpgen = FragmentCatalog.FragFPGenerator()
    for i, mol in enumerate(suppl):
      fp = fpgen.GetFPForMol(mol, self.fragCat)
      for bit in ranges[i]:
        assert fp[bit], '%s: %s' % (Chem.MolToSmiles(mol), str(bit))

  def test6Builder(self):
    suppl = Chem.SmilesMolSupplierFromText('\n'.join(self.smiList2), ',', 0, -1, 0)
    cat = BuildFragmentCatalog.BuildCatalog(suppl, minPath=1, reportFreq=20)
    assert cat.GetNumEntries() == 21
    assert cat.GetFPLength() == 21
    self._testBits(cat)

  def test7ScoreMolecules(self):
    suppl = Chem.SmilesMolSupplierFromText('\n'.join(self.smiList2), ',', 0, -1, 0)
    cat = BuildFragmentCatalog.BuildCatalog(suppl, minPath=1, reportFreq=20)
    assert cat.GetNumEntries() == 21
    assert cat.GetFPLength() == 21

    scores, obls = BuildFragmentCatalog.ScoreMolecules(suppl, cat, acts=self.list2Acts,
                                                       reportFreq=20)
    for i in range(len(self.list2Obls)):
      assert tuple(
        obls[i]) == self.list2Obls[i], '%d: %s != %s' % (i, str(obls[i]), str(self.list2Obls[i]))

    scores2 = BuildFragmentCatalog.ScoreFromLists(obls, suppl, cat, acts=self.list2Acts,
                                                  reportFreq=20)
    for i in range(len(scores)):
      assert (scores[i] == scores2[i]).all(), '%d: %s != %s' % (i, str(scores[i]), str(scores2[i]))

  def test8MolRanks(self):
    suppl = Chem.SmilesMolSupplierFromText('\n'.join(self.smiList2), ',', 0, -1, 0)
    cat = BuildFragmentCatalog.BuildCatalog(suppl, minPath=1, reportFreq=20)
    assert cat.GetNumEntries() == 21
    assert cat.GetFPLength() == 21

    # new InfoGain ranking:
    bitInfo, _ = BuildFragmentCatalog.CalcGains(suppl, cat, topN=10, acts=self.list2Acts,
                                                reportFreq=20, biasList=(1, ))
    entry = bitInfo[0]
    assert int(entry[0]) == 0
    assert cat.GetBitDescription(int(entry[0])) == 'C<-O>C'
    assert feq(entry[1], 0.4669)

    entry = bitInfo[1]
    assert int(entry[0]) in (2, 6)
    txt = cat.GetBitDescription(int(entry[0]))
    self.assertTrue(txt in ('C<-O>CC', 'C<-O>=C'), txt)
    assert feq(entry[1], 0.1611)

    entry = bitInfo[6]
    assert int(entry[0]) == 16
    assert cat.GetBitDescription(int(entry[0])) == 'C=CC<-O>'
    assert feq(entry[1], 0.0560)

    # standard InfoGain ranking:
    bitInfo, _ = BuildFragmentCatalog.CalcGains(suppl, cat, topN=10, acts=self.list2Acts,
                                                reportFreq=20)
    entry = bitInfo[0]
    assert int(entry[0]) == 0
    assert cat.GetBitDescription(int(entry[0])) == 'C<-O>C'
    assert feq(entry[1], 0.4669)

    entry = bitInfo[1]
    assert int(entry[0]) == 5
    assert cat.GetBitDescription(int(entry[0])) == 'C=CC'
    assert feq(entry[1], 0.2057)

  def test9Issue116(self):
    smiList = ['Cc1ccccc1']
    suppl = Chem.SmilesMolSupplierFromText('\n'.join(smiList), ',', 0, -1, 0)
    cat = BuildFragmentCatalog.BuildCatalog(suppl, minPath=2, maxPath=2)
    assert cat.GetFPLength() == 2
    assert cat.GetBitDescription(0) == 'ccC'
    fpgen = FragmentCatalog.FragFPGenerator()
    mol = Chem.MolFromSmiles('Cc1ccccc1')
    fp = fpgen.GetFPForMol(mol, cat)
    assert fp[0]
    assert fp[1]
    mol = Chem.MolFromSmiles('c1ccccc1-c1ccccc1')
    fp = fpgen.GetFPForMol(mol, cat)
    assert not fp[0]
    assert fp[1]


if __name__ == '__main__':
  unittest.main()



#
#  Copyright (C) 2001  greg Landrum
#
""" unit testing code for compound descriptors

"""

import math
import unittest

from rdkit.ML.Descriptors import Parser


class TestCase(unittest.TestCase):

  def setUp(self):
    self.piece1 = [['d1', 'd2', 's1'], ['d1', 'd2', 's1']]
    self.aDict = {'Fe': {'d1': 1, 'd2': 2, 's1': 'abc'}, 'Pt': {'d1': 10, 'd2': 20, 's1': 'def'}}
    self.pDict = {'d1': 100., 'd2': 200.}
    self.compos = [('Fe', 1), ('Pt', 1)]
    self.cExprs = [
      "SUM($1)", "SUM($1)+SUM($2)", "MEAN($1)", "DEV($2)", "MAX($1)", "MIN($2)", "SUM($1)/$a",
      'HAS($3,"def")', 'HAS($3,"xyz")', 'HAS($3)', "sqrt($a+($b+$b))"
    ]
    self.results = [11., 33., 5.5, 9., 10., 2., 0.11, 1, 0, -666, math.sqrt(500)]
    self.tol = 0.0001

  def testSingleCalcs(self):
    " testing calculation of a single descriptor "
    for i in range(len(self.cExprs)):
      cExpr = self.cExprs[i]
      argVect = self.piece1 + [cExpr]
      res = Parser.CalcSingleCompoundDescriptor(self.compos, argVect, self.aDict, self.pDict)
      self.assertAlmostEqual(res, self.results[i], 2)

  def testMultipleCalcs(self):
    " testing calculation of multiple descriptors "
    for i in range(len(self.cExprs)):
      cExpr = self.cExprs[i]
      argVect = self.piece1 + [cExpr]
      res = Parser.CalcMultipleCompoundsDescriptor([self.compos, self.compos], argVect, self.aDict,
                                                   [self.pDict, self.pDict])
      self.assertAlmostEqual(res[0], self.results[i], delta=self.tol,
                             msg='Expression {0} failed'.format(cExpr))
      self.assertAlmostEqual(res[1], self.results[i], delta=self.tol,
                             msg='Expression {0} failed'.format(cExpr))

  def _test_exampleCode(self):
    Parser._exampleCode()


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



#  Copyright (c) 2017 Greg Landrum
#  All rights reserved.
#
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.

import copy
import os
import sys
import unittest

from rdkit import Chem, Geometry
from rdkit.Chem import rdCoordGen, rdMolAlign


def compareConfs(c1, c2, match, tol=1e-2, alignIt=False):
  for i, j in enumerate(match):
    pi = c2.GetAtomPosition(i)
    pj = c1.GetAtomPosition(j)
    if (pj - pi).Length() >= tol:
      return False
  return True


class TestCase(unittest.TestCase):

  def test_basics(self):
    mol = Chem.MolFromSmiles('CCOC')
    self.assertEqual(mol.GetNumConformers(), 0)
    rdCoordGen.AddCoords(mol)
    self.assertEqual(mol.GetNumConformers(), 1)
    rdCoordGen.AddCoords(mol)
    self.assertEqual(mol.GetNumConformers(), 1)
    rwmol = Chem.RWMol(mol)
    rdCoordGen.AddCoords(rwmol)
    self.assertEqual(rwmol.GetNumConformers(), 1)

  def test_template1(self):
    template = Chem.MolFromSmiles('C1OCOCOCOCCCNCCC1')
    template.SetProp("_Name", 'template')
    mol = Chem.MolFromSmiles('C1OCOCOCOCCCNC(OC(=O)C2CC2)CC1')
    mol.SetProp("_Name", "mol")
    rdCoordGen.AddCoords(template)
    rdCoordGen.AddCoords(mol)
    self.assertFalse(
      compareConfs(mol.GetConformer(), template.GetConformer(), mol.GetSubstructMatch(template)))
    match = mol.GetSubstructMatch(template)
    mapd = dict()
    for i, aid in enumerate(match):
      p = template.GetConformer().GetAtomPosition(i)
      mapd[aid] = Geometry.Point2D(p.x, p.y)
    ps = rdCoordGen.CoordGenParams()
    ps.SetCoordMap(mapd)
    ps.dbg_useFixed = True
    rdCoordGen.AddCoords(mol, ps)
    self.assertTrue(
      compareConfs(mol.GetConformer(), template.GetConformer(), mol.GetSubstructMatch(template)))

  def test_template2(self):
    # the easier way...
    template = Chem.MolFromSmiles('C1OCOCOCOCCCNCCC1')
    template.SetProp("_Name", 'template')
    mol = Chem.MolFromSmiles('C1OCOCOCOCCCNC(OC(=O)C2CC2)CC1')
    mol.SetProp("_Name", "mol")
    rdCoordGen.AddCoords(template)
    ps = rdCoordGen.CoordGenParams()
    ps.SetTemplateMol(template)
    ps.dbg_useFixed = True
    rdCoordGen.AddCoords(mol, ps)
    self.assertTrue(
      compareConfs(mol.GetConformer(), template.GetConformer(), mol.GetSubstructMatch(template)))

  def test_template3(self):
    # the easier way, test lifetime...
    template = Chem.MolFromSmiles('C1OCOCOCOCCCNCCC1')
    mol = Chem.MolFromSmiles('C1OCOCOCOCCCNC(OC(=O)C2CC2)CC1')
    rdCoordGen.AddCoords(template)
    template2 = Chem.Mol(template)
    ps = rdCoordGen.CoordGenParams()
    ps.SetTemplateMol(template2)
    template2 = None
    ps.dbg_useFixed = True
    rdCoordGen.AddCoords(mol, ps)
    self.assertTrue(
      compareConfs(mol.GetConformer(), template.GetConformer(), mol.GetSubstructMatch(template)))


if __name__ == '__main__':
  unittest.main()



"""
2019, Daniel Probst, Reymond Group @ University of Bern
 @@ All Rights Reserved @@
This file is part of the RDKit.
The contents are covered by the terms of the BSD license
which is included in the file license.txt, found at the root
of the RDKit source tree.
"""

import unittest

from rdkit import Chem
from rdkit.Chem import rdMHFPFingerprint


class TestCase(unittest.TestCase):

  def setUp(self):
    pass

  def testMHFPDistance(self):
    s = "CN1C=NC2=C1C(=O)N(C(=O)N2C)C"
    enc = rdMHFPFingerprint.MHFPEncoder(128, 42)
    fp = enc.EncodeSmiles(s)
    dist = enc.Distance(fp, fp)
    self.assertEqual(dist, 0.0)

  def testMHFPFingerprint(self):
    s = "CN1C=NC2=C1C(=O)N(C(=O)N2C)C"
    t = "Cn1cnc2c1c(=O)[nH]c(=O)n2C"

    m = Chem.MolFromSmiles(s)
    enc = rdMHFPFingerprint.MHFPEncoder(128, 42)

    self.assertEqual(len(enc.CreateShinglingFromSmiles(s, rings=False)), 42)
    self.assertEqual(len(enc.CreateShinglingFromSmiles(s, min_radius=0)), 58)

    sh_a = enc.CreateShinglingFromSmiles(s)
    sh_b = enc.CreateShinglingFromMol(m)

    self.assertEqual(len(sh_a), 44)
    self.assertEqual(list(sh_a), list(sh_b))

    fp_a = enc.EncodeSmiles(s)
    fp_b = enc.EncodeMol(m)

    self.assertEqual(list(fp_a), list(fp_b))

    fp_c = enc.EncodeSmiles(t)
    dist = rdMHFPFingerprint.MHFPEncoder.Distance(fp_a, fp_c)
    self.assertEqual(dist, 0.5390625)


if __name__ == "__main__":
  unittest.main()



# $Id$
#
#  Copyright (C) 2002-2006  greg Landrum and Rational Discovery LLC
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
"""unit testing code for the EState atom typing

validation values are from the paper (JCICS _35_ 1039-1045 (1995))

"""

import unittest

from rdkit import Chem
from rdkit.Chem.EState import AtomTypes


class TestCase(unittest.TestCase):

  def _validate(self, vals, tol=1e-2, show=False):
    for smi, ans in vals:
      mol = Chem.MolFromSmiles(smi)
      types = AtomTypes.TypeAtoms(mol)
      if show:  # pragma: nocover
        print(types)
      self.assertEqual(len(ans), len(types), 'bad type len for smiles: %s' % (smi))
      lens = [len(x) for x in types]
      self.assertEqual(max(lens), 1, 'atom matched multiple types for smiles: %s' % (smi))
      for a, b in zip(ans, [x[0] for x in types]):
        self.assertEqual(a, b, 'bad type for SMILES: %s' % (smi))

  def test1_simpleMolecules(self):
    data = [
      ('CC', ['sCH3', 'sCH3']),
      ('CCC', ['sCH3', 'ssCH2', 'sCH3']),
      ('CCOC', ['sCH3', 'ssCH2', 'ssO', 'sCH3']),
      ('c1ccccc1[NH3+]', ['aaCH', 'aaCH', 'aaCH', 'aaCH', 'aaCH', 'aasC', 'sNH3']),
      ('c1ccccc1N', ['aaCH', 'aaCH', 'aaCH', 'aaCH', 'aaCH', 'aasC', 'sNH2']),
      ('C#C', ['tCH', 'tCH']),
      ('C=C=C', ['dCH2', 'ddC', 'dCH2']),
    ]
    self._validate(data, show=False)

  def test2_complexMolecules(self):
    data = [
      ('c1[nH]cnc1CC(N)C(O)=O',
       ['aaCH', 'aaNH', 'aaCH', 'aaN', 'aasC', 'ssCH2', 'sssCH', 'sNH2', 'dssC', 'sOH', 'dO']),
      ('c1nc[n-]c1CC(N)C(O)=O',
       ['aaCH', 'aaN', 'aaCH', 'aaN', 'aasC', 'ssCH2', 'sssCH', 'sNH2', 'dssC', 'sOH', 'dO']),
      ('c1cccc2c1cccc2',
       ['aaCH', 'aaCH', 'aaCH', 'aaCH', 'aaaC', 'aaaC', 'aaCH', 'aaCH', 'aaCH', 'aaCH']),
    ]
    self._validate(data, show=False)


if __name__ == '__main__':
  unittest.main()



import moda
import modb

ca = moda.ClassA()
print('ca:', ca.Get4())

cb = modb.ClassB()
print('cb:', cb.Get3())

newca = cb.ReturnOther()
print('new:', newca.Get4())

print('arg:', cb.AcceptOther(newca))



#
#  Copyright (C) 2000  greg Landrum
#
""" unit testing code for trees and decision trees (not learning/xvalidation) """

import unittest

from rdkit.ML.DecTree import TreeUtils
from rdkit.ML.DecTree.DecTree import DecTreeNode as Node


class TestTreeUtils(unittest.TestCase):

  def test_TreeUtils(self):
    # Tree is d1(d2,d3(d2,d4))
    t1 = Node(None, 'd1', 1)
    t2 = Node(None, 'd2', 2)
    t1.AddChildNode(t2)
    t2 = Node(None, 'd3', 3)
    t1.AddChildNode(t2)
    t3 = Node(None, 'd4', 4)
    t2.AddChildNode(t3)
    t3 = Node(None, 'd2', 2)
    t2.AddChildNode(t3)

    r = TreeUtils.CollectLabelLevels(t1, {})
    self.assertEqual(r, {1: 0, 2: 1, 3: 1, 4: 2})

    # Only to depth 2
    r = TreeUtils.CollectLabelLevels(t1, {}, 0, 2)
    self.assertEqual(r, {1: 0, 2: 1, 3: 1})

    # Check that we can handle subtrees:
    r = TreeUtils.CollectLabelLevels(t1, {}, 1, 2)
    self.assertEqual(r, {1: 1})

    names = TreeUtils.CollectDescriptorNames(t1, {})
    self.assertEqual(names, {1: 'd1', 2: 'd2', 3: 'd3', 4: 'd4'})

    names = TreeUtils.CollectDescriptorNames(t1, {}, 0, 2)
    self.assertEqual(names, {1: 'd1', 2: 'd2', 3: 'd3'})

    names = TreeUtils.CollectDescriptorNames(t1, {}, 1, 2)
    self.assertEqual(names, {1: 'd1'})


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



tests = [
  ("testExecs/test1.exe", "", {}),
  ("python", "test_list.py", {
    "dir": "Wrap"
  }),
]

longTests = []
if __name__ == '__main__':
  import sys

  import TestRunner
  failed, tests = TestRunner.RunScript('test_list.py', 0, 1)
  sys.exit(len(failed))



#
#  Copyright (C) 2002-2008  greg Landrum and Rational Discovery LLC
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
"""unit testing code for the signatures

"""
import os.path
import unittest

from rdkit import Chem, RDConfig
from rdkit.Chem import ChemicalFeatures
from rdkit.Chem.Pharm2D import Generate, SigFactory


class TestCase(unittest.TestCase):

  def setUp(self):
    fdefFile = os.path.join(RDConfig.RDCodeDir, 'Chem', 'Pharm2D', 'test_data', 'BaseFeatures.fdef')
    featFactory = ChemicalFeatures.BuildFeatureFactory(fdefFile)
    self.factory = SigFactory.SigFactory(featFactory, minPointCount=2, maxPointCount=3,
                                         trianglePruneBins=False)
    self.factory.SetBins([(0, 2), (2, 5), (5, 8)])
    self.factory.Init()
    SigFactory._verbose = False

  def test1Sizes(self):
    self.factory.maxPointCount = 2
    self.factory.Init()
    sig = self.factory.GetSignature()
    self.assertEqual(len(sig), 45)

    self.factory.maxPointCount = 3
    self.factory.Init()
    sig = self.factory.GetSignature()
    self.assertEqual(len(sig), 990)

    self.factory.maxPointCount = 4
    self.factory.Init()
    sig = self.factory.GetSignature()
    self.assertEqual(len(sig), 18000)

  def test2BitIdx(self):
    data = [
      ((0, 0), [0], 0),
      ((0, 0), [2], 1),
      ((0, 0), [5], 2),
      ((0, 1), [5], 5),
      ((1, 1), [4], 16),
      ((1, 1), [7], 17),
      ((0, 0, 0), [1, 1, 1], 45),
      ((0, 0, 1), [1, 1, 1], 72),
      ((0, 0, 1), [1, 1, 3], 75),
      ((0, 0, 1), [3, 1, 1], 81),
      ((0, 0, 1), [3, 3, 1], 84),
    ]
    for tpl in data:
      patts, dists, bit = tpl
      idx = self.factory.GetBitIdx(patts, dists)
      self.assertEqual(bit, idx)

      cnt, feats, _ = self.factory.GetBitInfo(bit)
      self.assertEqual(cnt, len(patts))
      self.assertEqual(feats, patts)

  def test3BitIdx(self):
    # test 3 point p'cophore ids, you can never have too much of this stuff
    self.factory.SetBins(((0, 2), (2, 4), (4, 8)))
    self.factory.Init()
    self.assertEqual(self.factory.GetSigSize(), 990)
    probes = [
      ((0, 0, 0), (1, 3, 1), 54),
      ((0, 0, 0), (3, 1, 1), 54),
      ((0, 0, 0), (1, 1, 3), 54),
      ((0, 0, 0), (1, 3, 3), 57),
      ((0, 0, 1), (1, 3, 1), 75),
    ]
    for patts, dists, ans in probes:
      idx = self.factory.GetBitIdx(patts, dists)
      self.assertEqual(idx, ans)

      cnt, feats, _ = self.factory.GetBitInfo(ans)
      self.assertEqual(cnt, len(patts))
      self.assertEqual(feats, patts)

  def test4BitIdx(self):
    self.factory.trianglePruneBins = True
    self.factory.Init()
    sig = self.factory.GetSignature()
    self.assertEqual(len(sig), 885)

    probes = [
      ((0, 0, 0), (1, 3, 1), 52),
      ((0, 0, 0), (1, 1, 3), 52),
      ((0, 0, 0), (3, 1, 1), 52),
      ((0, 0, 0), (1, 3, 3), 55),
      ((0, 0, 1), (1, 3, 1), 71),
    ]
    for patts, dists, ans in probes:
      idx = self.factory.GetBitIdx(patts, dists)
      self.assertEqual(idx, ans)
      cnt, feats, _ = self.factory.GetBitInfo(ans)
      self.assertEqual(cnt, len(patts))
      self.assertEqual(feats, patts)

    # Test exception handlers
    self.assertRaises(NotImplementedError, self.factory.GetBitIdx, (0, 0, 0, 0), (1, 3, 1, 2))
    self.assertRaises(IndexError, self.factory.GetBitIdx, (0, ), (1, ))
    self.factory.maxPointCount = 2
    self.assertRaises(IndexError, self.factory.GetBitIdx, (0, 0, 0), (1, 3, 1))
    self.factory.maxPointCount = 3
    self.assertRaises(IndexError, self.factory.GetBitIdx, (0, -1, 0), (1, 3, 1))
    self.assertRaises(IndexError, self.factory.GetBitIdx, (0, self.factory._nFeats, 0), (1, 3, 1))

  def test5SimpleSig(self):
    factory = self.factory
    factory.SetBins([(1, 3), (3, 7), (7, 10)])
    factory.minPointCount = 2
    factory.maxPointCount = 3
    factory.Init()

    mol = Chem.MolFromSmiles('O=CCC=O')
    sig = Generate.Gen2DFingerprint(mol, factory)
    self.assertEqual(len(sig), 990)
    bs = tuple(sig.GetOnBits())
    self.assertEqual(bs, (1, ))

    mol = Chem.MolFromSmiles('O=CC(CC=O)CCC=O')
    sig = Generate.Gen2DFingerprint(mol, factory)
    self.assertEqual(len(sig), 990)
    bs = tuple(sig.GetOnBits())
    self.assertEqual(bs, (1, 2, 67))

  def test6SimpleSigCounts(self):
    factory = self.factory
    factory.SetBins([(1, 3), (3, 7), (7, 10)])
    factory.minPointCount = 2
    factory.maxPointCount = 3
    factory.useCounts = True
    factory.Init()

    mol = Chem.MolFromSmiles('O=CCC=O')
    sig = Generate.Gen2DFingerprint(mol, factory)
    self.assertEqual(sig.GetLength(), 990)
    cs = tuple(sig.GetNonzeroElements().items())
    self.assertEqual(cs, ((1, 1), ))

    mol = Chem.MolFromSmiles('O=CC(CC=O)CCC=O')
    sig = Generate.Gen2DFingerprint(mol, factory)
    self.assertEqual(sig.GetLength(), 990)
    elems = sig.GetNonzeroElements()
    bs = list(elems.keys())
    bs.sort()
    cs = [(x, elems[x]) for x in bs]
    self.assertEqual(tuple(cs), ((1, 2), (2, 1), (67, 1)))

  def test7SimpleSigSkip(self):
    factory = self.factory
    factory.SetBins([(1, 3), (3, 7), (7, 10)])
    factory.minPointCount = 2
    factory.maxPointCount = 3
    factory.skipFeats = 'Acceptor'
    factory.Init()

    mol = Chem.MolFromSmiles('O=CCC=O')
    sig = Generate.Gen2DFingerprint(mol, factory)
    self.assertEqual(len(sig), 570)
    bs = tuple(sig.GetOnBits())
    self.assertEqual(bs, ())

  def test8MultiPointMatches(self):
    factory = self.factory
    factory.SetBins([(1, 3), (3, 7), (7, 10)])
    factory.minPointCount = 2
    factory.maxPointCount = 3
    factory.Init()

    mol = Chem.MolFromSmiles('O=Cc1ccccc1')
    sig = Generate.Gen2DFingerprint(mol, factory)
    self.assertEqual(len(sig), 990)
    bs = tuple(sig.GetOnBits())
    self.assertEqual(bs, (3, ))

    mol = Chem.MolFromSmiles('O=CCCCCCCCCc1ccccc1')
    sig = Generate.Gen2DFingerprint(mol, factory)
    self.assertEqual(len(sig), 990)
    bs = tuple(sig.GetOnBits())
    self.assertEqual(bs, ())

  # FIX: add test for perms argument to Gen2DFingerprint

  def test9BondOrderSigs(self):
    # test sigs where bond order is used
    factory = self.factory
    factory.SetBins([(1, 4), (4, 7), (7, 10)])
    factory.minPointCount = 2
    factory.maxPointCount = 3
    factory.Init()

    mol = Chem.MolFromSmiles('[O-]CCC(=O)')
    sig = Generate.Gen2DFingerprint(mol, self.factory)
    self.assertEqual(len(sig), 990)
    bs = tuple(sig.GetOnBits())
    self.assertEqual(bs, (1, ))

    self.factory.includeBondOrder = True
    sig = Generate.Gen2DFingerprint(mol, self.factory)
    self.assertEqual(len(sig), 990)
    bs = tuple(sig.GetOnBits())
    self.assertEqual(bs, (0, ))

  def testDefaultFactory(self):
    from rdkit.Chem import Pharm2D
    factory = Pharm2D.DefaultSigFactory()
    self.assertEqual(factory.GetNumBins(), 7)

    # Generate._verbose=True
    mol = Chem.MolFromSmiles('OCCC(=O)')
    sig = Generate.Gen2DFingerprint(mol, factory)
    self.assertEqual(len(sig), 19355)
    self.assertEqual(tuple(sig.GetOnBits()), (
      2,
      16,
      21,
      84,
      1274,
      4361,
    ))
    nPts, combo, scaffold, labels, dMat = factory._GetBitSummaryData(21)
    self.assertEqual(nPts, 2)
    self.assertEqual(labels, ['Acceptor', 'Hydrophobe'])
    self.assertEqual(list(dMat[0]), [0, 0])
    self.assertEqual(list(dMat[1]), [0, 0])

    txt = factory.GetBitDescription(21)
    self.assertEqual(txt, 'Acceptor Hydrophobe |0 0|0 0|')

    txt = factory.GetBitDescription(21)
    self.assertEqual(txt, 'Acceptor Hydrophobe |0 0|0 0|')

    nPts, combo, scaffold, labels, dMat = factory._GetBitSummaryData(2)
    self.assertEqual(nPts, 2)
    self.assertEqual(labels, ['Acceptor', 'Acceptor'])
    self.assertEqual(list(dMat[0]), [0, 2])
    self.assertEqual(list(dMat[1]), [2, 0])

    nPts, combo, scaffold, labels, dMat = factory._GetBitSummaryData(4361)
    self.assertEqual(nPts, 3)
    self.assertEqual(labels, ['Acceptor', 'Donor', 'Hydrophobe'])
    self.assertEqual(list(dMat[0]), [0, 2, 0])
    self.assertEqual(list(dMat[1]), [2, 0, 0])
    self.assertEqual(list(dMat[2]), [0, 0, 0])
    self.assertEqual(factory.GetBitDescription(4361),
                     'Acceptor Donor Hydrophobe |0 2 0|2 0 0|0 0 0|')

    self.assertRaises(NotImplementedError, factory.GetBitDescriptionAsText, 21)


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



"""
unit testing code rdkit.Chem.Pharm2D.Generate

"""
import unittest

from rdkit.Chem.Pharm2D import Generate


class TestCase(unittest.TestCase):

  def test_Gen2DFingerprint(self):
    # The code is run from most of the other tests. This is only
    # to test the assertions
    self.assertRaises(ValueError, Generate.Gen2DFingerprint, 'mol', 'incorrectType')


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



import doctest
import os.path
import unittest
from collections import namedtuple

from rdkit import Chem, RDConfig
from rdkit.Chem import QED

doLong = False
TestData = namedtuple('TestData', 'lineNo,smiles,mol,expected')
dataNCI200 = os.path.join(RDConfig.RDCodeDir, 'Chem', 'test_data', 'QED', 'NCI_200_qed.csv')
dataRegression = os.path.join(RDConfig.RDCodeDir, 'Chem', 'test_data', 'QED', 'Regression_qed.csv')


def load_tests(loader, tests, ignore):
  """ Add the Doctests from the module """
  tests.addTests(doctest.DocTestSuite(QED, optionflags=doctest.ELLIPSIS))
  return tests


class TestCase(unittest.TestCase):

  def testQED(self):
    self.assertEqual(QED.qed.version, '1.1.0',
                     msg='QED version has changed. Update the regression tests if required.')

  def testNCI200(self):
    for d in readTestData(dataNCI200):
      self.assertAlmostEqual(QED.qed(d.mol), d.expected,
                             msg='QED not equal to expected in line {}'.format(d.lineNo))
      # Check that adding hydrogens will not change the result
      # This is currently not the case. Hydrogens change the number of rotatable bonds and the
      # number of alerts.
      mol = Chem.AddHs(d.mol)
      self.assertAlmostEqual(QED.qed(mol), d.expected,
                             msg='QED not equal to expected in line {}'.format(d.lineNo))

  def testRegression(self):
    if not doLong:
      raise unittest.SkipTest('long test')
    for d in readTestData(dataRegression):
      self.assertAlmostEqual(QED.qed(d.mol), d.expected,
                             msg='QED not equal to expected in line {}'.format(d.lineNo))

  def test_properties(self):
    m = Chem.MolFromSmiles('N=C(CCSCc1csc(N=C(N)N)n1)NS(N)(=O)=O')
    p = QED.properties(m)
    self.assertAlmostEqual(p.MW, 337.456)
    self.assertAlmostEqual(p.ALOGP, -0.55833)
    self.assertAlmostEqual(p.HBA, 6)
    self.assertAlmostEqual(p.HBD, 5)
    self.assertAlmostEqual(p.PSA, 173.33)
    self.assertAlmostEqual(p.ROTB, 7)
    self.assertAlmostEqual(p.AROM, 1)
    self.assertAlmostEqual(p.ALERTS, 3)

    p = QED.properties(Chem.AddHs(m))
    self.assertAlmostEqual(p.MW, 337.456)
    self.assertAlmostEqual(p.ALOGP, -0.55833)
    self.assertAlmostEqual(p.HBA, 6)
    self.assertAlmostEqual(p.HBD, 5)
    self.assertAlmostEqual(p.PSA, 173.33)
    self.assertAlmostEqual(p.ROTB, 7)
    self.assertAlmostEqual(p.AROM, 1)
    self.assertAlmostEqual(p.ALERTS, 3)

  def test_examples(self):
    # Paroxetine 0.935
    self.assertAlmostEqual(QED.qed(Chem.MolFromSmiles('c1cc2OCOc2cc1OCC1CNCCC1c1ccc(F)cc1')), 0.934,
                           places=3)
    # Leflunomide 0.929
    self.assertAlmostEqual(QED.qed(Chem.MolFromSmiles('C1=NOC(C)=C1C(=O)Nc1ccc(cc1)C(F)(F)F')),
                           0.911, places=3)
    # Clomipramine 0.779
    self.assertAlmostEqual(QED.qed(Chem.MolFromSmiles('CN(C)CCCN1c2ccccc2CCc2ccc(Cl)cc21')), 0.818,
                           places=3)
    # Tegaserod 0.213
    self.assertAlmostEqual(QED.qed(Chem.MolFromSmiles('CCCCCNC(=N)NN=CC1=CNc2ccc(CO)cc21')), 0.235,
                           places=3)


def readTestData(filename):
  """ Read test data from file """
  with open(filename, 'r') as f:
    for lineNo, line in enumerate(f, 1):
      if line[0] == '#':
        continue
      smiles, expected = line.strip().split(',')
      mol = Chem.MolFromSmiles(smiles)
      if not mol:
        raise AssertionError('molecule construction failed on line %d' % lineNo)
      yield TestData(lineNo, smiles, mol, float(expected))


def updateTestData():
  """ Update the test data. This should only be done if the method changes! """
  for filename in (
      dataNCI200,
      dataRegression,
  ):
    data = list(readTestData(filename))
    with open(filename, 'w') as f:
      print('# Test data for QED descriptor', file=f)
      for d in data:
        expected = QED.qed(d.mol)
        print('{0.smiles},{1}'.format(d, expected), file=f)


if __name__ == '__main__':  # pragma: nocover
  import argparse
  import sys
  parser = argparse.ArgumentParser()
  parser.add_argument('-l', default=False, action='store_true', dest='doLong')
  parser.add_argument('-u', default=False, action='store_true', dest='updateTestData')
  args = parser.parse_args()

  # Handle possible arguments
  doLong = args.doLong
  if args.doLong:
    sys.argv.remove('-l')

  if args.updateTestData:
    updateTestData()
    sys.argv.remove('-u')

  unittest.main()



tests = [
  ("python", "UnitTestSDFToCSV.py", {}),
]

longTests = []

if __name__ == '__main__':
  import sys

  from rdkit import TestRunner
  doLong = 0
  if '-l' in sys.argv:
    doLong = 1
  failed, tests = TestRunner.RunScript('test_list.py', doLong, 1)
  sys.exit(len(failed))



import logging
import sys

# Need to import rdBase to properly wrap exceptions
# otherwise they will leak memory
from . import rdBase

try:
  from .rdBase import rdkitVersion as __version__
except ImportError:
  __version__ = 'Unknown'
  raise

logger = logging.getLogger("rdkit")

# if we are running in a jupyter notebook, enable the extensions
try:
  kernel_name = get_ipython().__class__.__name__
  module_name = get_ipython().__class__.__module__

  if kernel_name == 'ZMQInteractiveShell' or module_name == 'google.colab._shell':
    logger.info("Enabling RDKit %s jupyter extensions" % __version__)
    from rdkit.Chem.Draw import IPythonConsole
    rdBase.LogToPythonStderr()
except Exception:
  pass

# Do logging setup at the end, so users can suppress the
# "enabling jupyter" message at the root logger.
log_handler = logging.StreamHandler(sys.stderr)
logger.addHandler(log_handler)
logger.setLevel(logging.WARN)
logger.propagate = False

# Uncomment this to use Python logging by default:
# rdBase.LogToPythonLogger()



#
#  Copyright (C) 2002-2008  greg Landrum and Rational Discovery LLC
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
import os.path
import unittest
from io import StringIO

from rdkit import Chem, RDConfig
from rdkit.Chem import ChemicalFeatures
from rdkit.Chem.Pharm2D import Generate, Gobbi_Pharm2D, Matcher, SigFactory
from rdkit.TestRunner import redirect_stdout


class TestCase(unittest.TestCase):

  def setUp(self):
    fdefFile = os.path.join(RDConfig.RDCodeDir, 'Chem', 'Pharm2D', 'test_data', 'BaseFeatures.fdef')
    featFactory = ChemicalFeatures.BuildFeatureFactory(fdefFile)
    self.factory = SigFactory.SigFactory(featFactory, minPointCount=2, maxPointCount=3)
    self.factory.SetBins([(0, 2), (2, 5), (5, 8)])
    self.factory.Init()

  def test1_simple(self):
    mol = Chem.MolFromSmiles('OCC(=O)CCCN')
    self.factory.skipFeats = ['Donor']
    self.factory.Init()
    self.assertEqual(self.factory.GetSigSize(), 510)
    Generate._verbose = False
    sig = Generate.Gen2DFingerprint(mol, self.factory)
    Generate._verbose = False
    tgt = (1, 2, 11, 52, 117)
    onBits = sig.GetOnBits()
    self.assertEqual(tuple(onBits), tgt)
    self.assertEqual(len(onBits), len(tgt))

    bitMatches = (
      [((0, ), (3, ))],
      [((0, ), (7, )), ((3, ), (7, ))],
      [((0, ), (3, ), (7, ))],
    )
    for i, bit in enumerate(onBits):
      matches = Matcher.GetAtomsMatchingBit(self.factory, bit, mol)
      # print bit,matches
      # tgt = bitMatches[i]
      # self.assertEqual(matches,tgt)

  def test2Bug28(self):
    smi = r'Cc([s]1)nnc1SCC(\CS2)=C(/C([O-])=O)N3C(=O)[C@H]([C@@H]23)NC(=O)C[n]4cnnn4'
    mol = Chem.MolFromSmiles(smi)
    factory = Gobbi_Pharm2D.factory
    factory.SetBins([(2, 3), (3, 4), (4, 5), (5, 8), (8, 100)])
    sig = Generate.Gen2DFingerprint(mol, factory)
    onBits = sig.GetOnBits()
    for bit in onBits:
      atoms = Matcher.GetAtomsMatchingBit(factory, bit, mol, justOne=1)
      self.assertTrue(len(atoms))

  def test3Roundtrip(self):
    # longer-running Bug 28 test
    nToDo = 20
    with open(os.path.join(RDConfig.RDDataDir, 'NCI', 'first_5K.smi'), 'r') as inF:
      inD = inF.readlines()[:nToDo]
    factory = Gobbi_Pharm2D.factory
    factory.SetBins([(2, 3), (3, 4), (4, 5), (5, 8), (8, 100)])
    for line in inD:
      smi = line.split('\t')[0]
      mol = Chem.MolFromSmiles(smi)
      sig = Generate.Gen2DFingerprint(mol, factory)
      onBits = sig.GetOnBits()
      for bit in onBits:
        atoms = Matcher.GetAtomsMatchingBit(factory, bit, mol, justOne=1)
        assert len(atoms), f'bit {bit} failed to match for smi {smi}'

  def test_exampleCode(self):
    # We make sure that the example code runs
    f = StringIO()
    with redirect_stdout(f):
      Matcher._exampleCode()
    self.assertIn('finished', f.getvalue())


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



# -*- coding: utf-8 -*-
"""
molvs.validate
~~~~~~~~~~~~~~

This module contains the main :class:`~molvs.validate.Validator` class that can be used to perform all
:class:`Validations <molvs.validations.Validation>`, as well as the :func:`~molvs.validate.validate_smiles()`
convenience function.

:copyright: Copyright 2016 by Matt Swain.
:license: MIT, see LICENSE file for more details.
"""
from warnings import warn

warn(f'The module {__name__} is deprecated and will be removed in the next release.',
     DeprecationWarning, stacklevel=2)

import logging
import sys

from rdkit import Chem

from .errors import StopValidateError
from .validations import VALIDATIONS

#: The default format for log messages.
SIMPLE_FORMAT = '%(levelname)s: [%(validation)s] %(message)s'

#: A more detailed format for log messages. Specify when initializing a Validator.
LONG_FORMAT = '%(asctime)s - %(levelname)s - %(validation)s - %(message)s'


class LogHandler(logging.Handler):
  """A simple logging Handler that just stores logs in an array until flushed."""

  def __init__(self):
    logging.Handler.__init__(self)
    self.logs = []

  @property
  def logmessages(self):
    return [self.format(record) for record in self.logs]

  def emit(self, record):
    """Append the record."""
    self.logs.append(record)

  def flush(self):
    """Clear the log records."""
    self.acquire()
    try:
      self.logs = []
    finally:
      self.release()

  def close(self):
    """Close the handler."""
    self.flush()
    logging.Handler.close(self)


class Validator(object):
  """The main class for running :class:`Validations <molvs.validations.Validation>` on molecules."""

  def __init__(self, validations=VALIDATIONS, log_format=SIMPLE_FORMAT, level=logging.INFO,
               stdout=False, raw=False):
    """Initialize a Validator with the following parameters:

        :param validations: A list of Validations to apply (default: :data:`~molvs.validations.VALIDATIONS`).
        :param string log_format: A string format (default: :data:`~molvs.validate.SIMPLE_FORMAT`).
        :param level: The minimum logging level to output.
        :param bool stdout: Whether to send log messages to standard output.
        :param bool raw: Whether to return raw :class:`~logging.LogRecord` objects instead of formatted log strings.
        """
    warn(
      f'The class {self.__class__.__name__} is deprecated and will be removed in the next release.',
      DeprecationWarning, stacklevel=2)
    self.raw = raw
    # Set up logger and add default LogHandler
    self.log = logging.getLogger(type(self).__name__)
    self.log.setLevel(level)
    self.handler = LogHandler()
    self.handler.setFormatter(logging.Formatter(log_format))
    self.log.addHandler(self.handler)
    # Add stdout StreamHandler if specified in parameters
    if stdout:
      strhdlr = logging.StreamHandler(sys.stdout)
      strhdlr.setFormatter(logging.Formatter(log_format))
      self.log.addHandler(strhdlr)
    # Instantiate the validations
    self.validations = [validation(self.log) for validation in validations]

  def __call__(self, mol):
    """Calling a Validator instance like a function is the same as calling its
        :meth:`~molvs.validate.Validator.validate` method."""
    return self.validate(mol)

  def validate(self, mol):
    """"""
    # Clear any log messages from previous runs
    self.handler.flush()
    # Run every validation, stopping if StopValidateError is raised
    for validation in self.validations:
      try:
        validation(mol)
      except StopValidateError:
        break
    return self.handler.logs if self.raw else self.handler.logmessages


def validate_smiles(smiles):
  """Return log messages for a given SMILES string using the default validations.

    Note: This is a convenience function for quickly validating a single SMILES string. It is more efficient to use
    the :class:`~molvs.validate.Validator` class directly when working with many molecules or when custom options
    are needed.

    :param string smiles: The SMILES for the molecule.
    :returns: A list of log messages.
    :rtype: list of strings.
    """
  # Skip sanitize as standardize does this anyway
  warn(f'The function validate_smiles is deprecated and will be removed in the next release.',
     DeprecationWarning, stacklevel=2)

  mol = Chem.MolFromSmiles(smiles)
  logs = Validator().validate(mol)
  return logs



#
#  Copyright (C) 2016  greg Landrum
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
""" unit testing code for IPython/Jupyter integration
"""
import unittest

from rdkit import Chem
from rdkit.Chem import Draw

try:
  from IPython.core.display import SVG

  from rdkit.Chem.Draw import IPythonConsole, rdMolDraw2D
except ImportError:
  IPythonConsole = None


class TestCase(unittest.TestCase):

  def setUp(self):
    if IPythonConsole is not None and Draw.MolsToGridImage != IPythonConsole.ShowMols:
      IPythonConsole.InstallIPythonRenderer()
    self.mol = Chem.MolFromSmiles('c1c(C[15NH3+])ccnc1[C@](Cl)(Br)[C@](Cl)(Br)F')

  def tearDown(self):
    if IPythonConsole is not None:
      IPythonConsole.UninstallIPythonRenderer()

  @unittest.skipIf(IPythonConsole is None, 'IPython not available')
  def testGithub1089(self):
    m = Chem.MolFromSmiles('CCCC')

    IPythonConsole.ipython_useSVG = False
    res = Draw.MolsToGridImage((m, m))
    self.assertFalse(isinstance(res, SVG))

    res = Draw.MolsToGridImage((m, m), useSVG=True)
    self.assertTrue(isinstance(res, SVG))

    IPythonConsole.ipython_useSVG = True
    res = Draw.MolsToGridImage((m, m))
    self.assertTrue(isinstance(res, SVG))

  @unittest.skipIf(IPythonConsole is None, 'IPython not available')
  def testGithub3101(self):
    m = Chem.MolFromSmiles('CCCC')

    IPythonConsole.ipython_useSVG = True
    IPythonConsole.drawOptions.addAtomIndices = True
    res = Draw.MolsToGridImage((m, m))
    self.assertIn('class="note"', res.data)

    dopts = rdMolDraw2D.MolDrawOptions()
    res = Draw.MolsToGridImage((m, m), drawOptions=dopts)
    self.assertNotIn('class="note"', res.data)

  @unittest.skipIf(IPythonConsole is None, 'IPython not available')
  def testMolPropertyRendering(self):
    m = Chem.MolFromSmiles('CCCC')
    self.assertTrue(hasattr(m, '_repr_html_'))

    m.SetProp('_Name', 'testm')
    m.SetProp('_hiddenprop', 'hpvalue')
    m.SetProp('computedprop', 'cpvalue', computed=True)
    m.SetDoubleProp('foo', 12.3)
    m.SetProp('publicprop', 'ppropval')

    IPythonConsole.ipython_showProperties = True
    html = m._repr_html_()
    self.assertIn('publicprop', html)
    self.assertNotIn('testm', html)
    self.assertNotIn('_hiddenprop', html)
    self.assertNotIn('computedprop', html)

    IPythonConsole.ipython_showProperties = False
    html = m._repr_html_()
    self.assertNotIn('publicprop', html)
    self.assertNotIn('_hiddenprop', html)
    self.assertNotIn('computedprop', html)

    for i in range(10):
      m.SetIntProp(f'prop-{i}', i)
    IPythonConsole.ipython_showProperties = True
    IPythonConsole.ipython_maxProperties = 5
    html = m._repr_html_()
    self.assertIn('publicprop', html)
    self.assertIn('prop-1', html)
    self.assertNotIn('prop-8', html)

    IPythonConsole.ipython_maxProperties = -1
    html = m._repr_html_()
    self.assertIn('publicprop', html)
    self.assertIn('prop-1', html)
    self.assertIn('prop-8', html)

    IPythonConsole.ipython_maxProperties = 10
    html = m._repr_html_()
    self.assertIn('publicprop', html)
    self.assertIn('prop-1', html)
    self.assertNotIn('prop-8', html)

  @unittest.skipIf(IPythonConsole is None, 'IPython not available')
  def testMolsMatrixToGridImage(self):
    # For testMolsMatrixToLinear and testMolsMatrixToLinear (which test MolsMatrixToGridImage and its helper _MolsNestedToLinear)
    s = "NC(C)C(=O)"
    mol = Chem.MolFromSmiles(s)
    # Set up matrix with oligomer count for the molecules
    # Should produce this grid:
    # NC(C)C(=O)
    #                                NC(C)C(=O)NC(C)C(=O)
    # NC(C)C(=O)NC(C)C(=O)NC(C)C(=O)                      NC(C)C(=O)NC(C)C(=O)NC(C)C(=O)NC(C)C(=O)
    repeats = [[1], [0, 2], [3, 0, 4]]

    # Create molecule if there are 1 or more oligomers;
    # otherwise, use None for molecule because drawing is distorted if use Chem.MolFromSmiles("")
    molsMatrix = [[Chem.MolFromSmiles(s * count) if count else None for count in row]
                  for row in repeats]

    legendsMatrix = [[str(count) + " unit(s)" for count in row] for row in repeats]
    d = Draw.MolsMatrixToGridImage(molsMatrix, legendsMatrix=legendsMatrix)
    self.assertIsNotNone(d)


if __name__ == '__main__':
  unittest.main()
